@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -35,10 +35,13 @@
 static GConfEngine *
 get_gconf_engine(void)
 {
+	GStaticMutex lock = G_STATIC_MUTEX_INIT;
 	/* Initialize *engine first time we're called. Otherwise just return the one we've got. */
+	g_static_mutex_lock(&lock);
 	static GConfEngine *engine = NULL;
 	if (!engine)
 		engine = gconf_engine_get_default();
+	g_static_mutex_unlock(&lock);
 	return engine;
 }
 #else
@@ -66,7 +69,8 @@ rs_conf_get_boolean(const gchar *name, gboolean *boolean_value)
 			if (gvalue->type == GCONF_VALUE_BOOL)
 			{
 				ret = TRUE;
-				*boolean_value = gconf_value_get_bool(gvalue);
+				if (boolean_value)
+					*boolean_value = gconf_value_get_bool(gvalue);
 			}
 			gconf_value_free(gvalue);
 		}
@@ -83,7 +87,8 @@ gboolean
 rs_conf_get_boolean_with_default(const gchar *name, gboolean *boolean_value, gboolean default_value)
 {
 	gboolean ret = FALSE;
-	*boolean_value = default_value;
+	if (boolean_value)
+		*boolean_value = default_value;
 #ifdef WITH_GCONF
 	GConfValue *gvalue;
 	GConfEngine *engine = get_gconf_engine();
@@ -97,7 +102,8 @@ rs_conf_get_boolean_with_default(const gchar *name, gboolean *boolean_value, gbo
 			if (gvalue->type == GCONF_VALUE_BOOL)
 			{
 				ret = TRUE;
-				*boolean_value = gconf_value_get_bool(gvalue);
+				if (boolean_value)
+					*boolean_value = gconf_value_get_bool(gvalue);
 			}
 			gconf_value_free(gvalue);
 		}
