@@ -24,6 +24,7 @@
 #define CONF_DEFAULT_EXPORT_TEMPLATE "default_export_template"
 #define CONF_CACHEDIR_IS_LOCAL "cache_in_home"
 #define CONF_LOAD_GDK "open_8bit_images"
+#define CONF_LOAD_RECURSIVE "load_recursive"
 #define CONF_PRELOAD "preload_photos"
 #define CONF_SAVE_FILETYPE "save_filetype"
 #define CONF_BATCH_DIRECTORY "batch_directory"
@@ -53,6 +54,7 @@
 #define CONF_ROI_GRID "roi_grid"
 #define CONF_CROP_ASPECT "crop_aspect"
 #define CONF_SHOW_FILENAMES "show_filenames_in_iconview"
+#define CONF_USE_SYSTEM_THEME "use_system_theme"
 #define CONF_FULLSCREEN "fullscreen"
 #define CONF_SHOW_TOOLBOX_FULLSCREEN "show_toolbox_fullscreen"
 #define CONF_SHOW_TOOLBOX "show_toolbox"
@@ -63,6 +65,7 @@
 #define CONF_SHOW_TOOLBOX_HUE "show_toolbox_hue"
 #define CONF_SHOW_TOOLBOX_CONTRAST "show_toolbox_contrast"
 #define CONF_SHOW_TOOLBOX_WARMTH "show_toolbox_warmth"
+#define CONF_SHOW_TOOLBOX_SHARPEN "show_toolbox_sharpen"
 #define CONF_SHOW_TOOLBOX_CURVE "show_toolbox_curve"
 #define CONF_SHOW_TOOLBOX_TRANSFORM "show_toolbox_transform"
 #define CONF_SHOW_TOOLBOX_HIST "show_toolbox_hist"
@@ -88,10 +91,12 @@
 #define DEFAULT_CONF_SHOW_TOOLBOX_HUE TRUE
 #define DEFAULT_CONF_SHOW_TOOLBOX_CONTRAST TRUE
 #define DEFAULT_CONF_SHOW_TOOLBOX_WARMTH TRUE
+#define DEFAULT_CONF_SHOW_TOOLBOX_SHARPEN TRUE
 #define DEFAULT_CONF_SHOW_TOOLBOX_CURVE TRUE
 #define DEFAULT_CONF_SHOW_TOOLBOX_TRANSFORM TRUE
 #define DEFAULT_CONF_SHOW_TOOLBOX_HIST TRUE
-
+#define DEFAULT_CONF_LOAD_RECURSIVE FALSE
+#define DEFAULT_CONF_USE_SYSTEM_THEME FALSE
 
 /* get the last working directory from gconf */
 void rs_set_last_working_directory(const char *lwd);
