@@ -42,6 +42,7 @@ struct _RS_CONFBOX
 
 static void gui_confbox_changed(GtkComboBox *filetype_combo, gpointer callback_data);
 static gboolean gui_confbox_deleted(GtkWidget *widget, GdkEvent *event, gpointer callback_data);
+static gboolean gui_confbox_select_value(RS_CONFBOX *confbox, gchar *value);
 static void gui_cms_in_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data);
 static void gui_cms_di_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data);
 static void gui_cms_ex_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data);
@@ -131,19 +132,14 @@ gui_confbox_add_entry(RS_CONFBOX *confbox, const gchar *conf_id, const gchar *te
 	return;
 }
 
-void
-gui_confbox_load_conf(RS_CONFBOX *confbox, gchar *default_value)
+gboolean
+gui_confbox_select_value(RS_CONFBOX *confbox, gchar *value)
 {
 	gchar *conf_id;
-	gchar *value;
-	gchar *needle = default_value;
 	GtkTreeModel *model;
 	GtkTreePath *path;
 	GtkTreeIter iter;
-
-	value = rs_conf_get_string(confbox->conf_key);
-	if (value)
-		needle = value;
+	gboolean found = FALSE;
 
 	model = gtk_combo_box_get_model(GTK_COMBO_BOX(confbox->widget));
 	path = gtk_tree_path_new_first();
@@ -152,15 +148,31 @@ gui_confbox_load_conf(RS_CONFBOX *confbox, gchar *default_value)
 
 	do {
 		gtk_tree_model_get(model, &iter, COMBO_CONF_ID, &conf_id, -1);
-		if (g_str_equal(conf_id, needle))
+		if (g_str_equal(conf_id, value))
 		{
 			gtk_combo_box_set_active_iter(GTK_COMBO_BOX(confbox->widget), &iter);
+			found = TRUE;
 			break;
 		}
 	} while(gtk_tree_model_iter_next (model, &iter));
 
+	return found;
+}
+
+void
+gui_confbox_load_conf(RS_CONFBOX *confbox, gchar *default_value)
+{
+	gchar *value;
+
+	value = rs_conf_get_string(confbox->conf_key);
 	if (value)
+	{
+		if (!gui_confbox_select_value(confbox, value))
+			gui_confbox_select_value(confbox, default_value);
 		g_free(value);
+	}
+	else
+		gui_confbox_select_value(confbox, default_value);
 	return;
 }
 
@@ -362,7 +374,6 @@ cms_enable_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 	rs_conf_set_boolean(CONF_CMS_ENABLED, togglebutton->active);
 	rs_cms_enable(rs->cms, togglebutton->active);
 	rs_preview_widget_set_cms(RS_PREVIEW_WIDGET(rs->preview), rs_cms_get_transform(rs->cms, PROFILE_DISPLAY));
-	rs_update_preview(rs);
 	return;
 }
 
@@ -375,7 +386,7 @@ gui_cms_in_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 	filename = rs_conf_get_cms_profile(RS_CMS_PROFILE_IN);
 	rs_cms_set_profile(rs->cms, PROFILE_INPUT, filename);
 	g_free(filename);
-	rs_update_preview(rs);
+	rs_preview_widget_update(RS_PREVIEW_WIDGET(rs->preview), FALSE);
 	return;
 }
 
@@ -389,7 +400,7 @@ gui_cms_di_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 	filename = rs_conf_get_cms_profile(RS_CMS_PROFILE_DISPLAY);
 	rs_cms_set_profile(rs->cms, PROFILE_DISPLAY, filename);
 	g_free(filename);
-	rs_update_preview(rs);
+	rs_preview_widget_update(RS_PREVIEW_WIDGET(rs->preview), FALSE);
 	return;
 }
 
@@ -403,7 +414,6 @@ gui_cms_ex_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 	filename = rs_conf_get_cms_profile(RS_CMS_PROFILE_EXPORT);
 	rs_cms_set_profile(rs->cms, PROFILE_EXPORT, filename);
 	g_free(filename);
-	rs_update_preview(rs);
 	return;
 }
 
@@ -414,7 +424,7 @@ gui_cms_intent_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 	gint active = gtk_combo_box_get_active(combobox);
 	rs_conf_set_cms_intent(CONF_CMS_INTENT, &active);
 	rs_cms_set_intent(rs->cms, active);
-	rs_update_preview(rs);
+	rs_preview_widget_update(RS_PREVIEW_WIDGET(rs->preview), FALSE);
 	return;
 }
 
@@ -433,12 +443,13 @@ gui_cms_choose_profile(const gchar *path)
 		GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 	gtk_dialog_set_default_response(GTK_DIALOG(fc), GTK_RESPONSE_ACCEPT);
 
-	if (path)
+	if (path && g_file_test(path, G_FILE_TEST_IS_DIR))
 		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(fc), path);
 	else
 	{
 #define DEAULT_PROFILE_PATH "/usr/share/color/icc"
-		if (g_access(DEAULT_PROFILE_PATH, X_OK)==0)
+		if ((g_access(DEAULT_PROFILE_PATH, X_OK)==0)
+			&& g_file_test(DEAULT_PROFILE_PATH, G_FILE_TEST_IS_DIR))
 			gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(fc),
 				DEAULT_PROFILE_PATH);
 #undef DEAULT_PROFILE_PATH
@@ -873,3 +884,47 @@ gui_menu_popup(GtkWidget *widget, gpointer user_data, ...)
 
 	return (menu);
 }
+
+void
+gui_select_theme(RS_THEME theme)
+{
+	static RS_THEME current_theme = STANDARD_GTK_THEME;
+	static gchar **default_rc_files = NULL;
+	GStaticMutex lock = G_STATIC_MUTEX_INIT;
+	GtkSettings *settings;
+
+	g_static_mutex_lock(&lock);
+
+	/* Copy default RC files */
+	if (!default_rc_files)
+	{
+		gchar **def;
+		gint i;
+		def = gtk_rc_get_default_files();
+		for(i=0;def[i];i++); /* Count */
+		default_rc_files = g_new0(gchar *, i+1);
+		for(i=0;def[i];i++) /* Copy */
+			default_rc_files[i] = g_strdup(def[i]);
+	}
+
+	/* Change theme if needed */
+	if (theme != current_theme)
+	{
+		settings = gtk_settings_get_default();
+		switch (theme)
+		{
+			case STANDARD_GTK_THEME:
+				gtk_rc_set_default_files(default_rc_files);
+				break;
+			case RAWSTUDIO_THEME:
+				gtk_rc_add_default_file(PACKAGE_DATA_DIR "/" PACKAGE "/rawstudio.gtkrc");
+				break;
+		}
+		current_theme = theme;
+		/* Reread everything */
+		if (settings)
+			gtk_rc_reparse_all_for_settings(settings, TRUE);
+	}
+
+	g_static_mutex_unlock(&lock);
+}
