@@ -28,6 +28,7 @@ struct _RS_PROGRESS {
 	GtkWidget *window;
 	GtkWidget *progressbar;
 	GtkWidget *frame;
+	GtkWidget *vbox;
 	gint items;
 	gint current;
 	const gchar *title;
@@ -47,7 +48,7 @@ gui_progress_destroy(GtkWidget *widget, GdkEvent *event, RS_PROGRESS *rsp)
  * @return A new RS_PROGRESS
  */
 static RS_PROGRESS *
-gui_progress_init()
+gui_progress_init(const gchar *title)
 {
 	extern GtkWindow *rawstudio_window;
 	GtkWidget *alignment;
@@ -62,20 +63,25 @@ gui_progress_init()
 	rsp->frame = gtk_frame_new(_("Progress"));
 	gtk_container_set_border_width (GTK_CONTAINER (rsp->frame), 5);
 
+	rsp->vbox = gtk_vbox_new(FALSE, 2);
+
 	rsp->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	g_signal_connect((gpointer) rsp->window, "delete_event", G_CALLBACK(gui_progress_destroy), rsp);
-	gtk_window_set_resizable(GTK_WINDOW(rsp->window), FALSE);
-	gtk_window_set_decorated(GTK_WINDOW(rsp->window), FALSE);
+	g_signal_connect(G_OBJECT(rsp->window), "delete_event", G_CALLBACK(gui_progress_destroy), rsp);
+	gtk_widget_realize (rsp->window);
+	gdk_window_set_type_hint(rsp->window->window, GDK_WINDOW_TYPE_HINT_UTILITY);
 	gtk_window_set_position(GTK_WINDOW(rsp->window), GTK_WIN_POS_CENTER_ON_PARENT);
-	gtk_window_set_title(GTK_WINDOW(rsp->window), _("Progress"));
+	gtk_window_set_title(GTK_WINDOW(rsp->window), title);
 	gtk_window_set_transient_for(GTK_WINDOW (rsp->window), rawstudio_window);
 
-	gtk_container_add (GTK_CONTAINER (rsp->window), rsp->frame);
+	gtk_box_pack_start(GTK_BOX(rsp->vbox), rsp->frame, TRUE, TRUE, 0);
+
+	gtk_container_add (GTK_CONTAINER (rsp->window), rsp->vbox);
 	gtk_container_add (GTK_CONTAINER (rsp->frame), alignment);
 	gtk_container_add (GTK_CONTAINER (alignment), rsp->progressbar);
 
 	rsp->items = 1;
 	rsp->current = 0;
+	rsp->title = NULL;
 	rsp->delay = -1.0f;
 	rsp->lifetime = NULL;
 	gui_progress_set_current(rsp, 0);
@@ -85,12 +91,12 @@ gui_progress_init()
 RS_PROGRESS *
 gui_progress_new(const gchar *title, gint items)
 {
-	RS_PROGRESS *rsp = gui_progress_init();
+	RS_PROGRESS *rsp = gui_progress_init(title);
 
 	if (items==0) items = 1;
 
 	if (title)
-		gtk_frame_set_label(GTK_FRAME(rsp->frame), title);
+		gtk_frame_set_label(GTK_FRAME(rsp->frame), _("Progress"));
 	rsp->items = items;
 	rsp->current = 0;
 	gui_progress_set_current(rsp, 0);
@@ -108,12 +114,12 @@ gui_progress_new(const gchar *title, gint items)
 RS_PROGRESS *
 gui_progress_new_with_delay(const gchar *title, gint items, gint delay)
 {
-	RS_PROGRESS *rsp = gui_progress_init();
+	RS_PROGRESS *rsp = gui_progress_init(title);
 
 	if (items==0) items = 1;
 
 	if (title)
-		gtk_frame_set_label(GTK_FRAME(rsp->frame), title);
+		gtk_frame_set_label(GTK_FRAME(rsp->frame), _("Progress"));
 	rsp->items = items;
 	rsp->current = 0;
 	rsp->delay = ((gdouble)delay)/1000.0f;
@@ -125,11 +131,13 @@ gui_progress_new_with_delay(const gchar *title, gint items, gint delay)
 void
 gui_progress_free(RS_PROGRESS *rsp)
 {
+	extern GtkWindow *rawstudio_window;
 	gtk_widget_destroy(rsp->window);
 	/* Free the GTimer if needed */
 	if (rsp->lifetime)
 		g_timer_destroy(rsp->lifetime);
 	g_free(rsp);
+	gtk_window_present(rawstudio_window);
 }
 
 void
@@ -148,7 +156,10 @@ gui_progress_set_current(RS_PROGRESS *rsp, gint current)
 
 	/* Show the widget if we're past the initial delay */
 	if ((rsp->delay>-1.0f) && (g_timer_elapsed(rsp->lifetime, NULL) > rsp->delay))
+	{
 		gtk_widget_show_all(rsp->window);
+		gtk_window_present(GTK_WINDOW(rsp->window));
+	}
 
 	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(rsp->progressbar),
 		((gdouble)rsp->current)/((gdouble)rsp->items));
@@ -168,3 +179,15 @@ gui_progress_set_current(RS_PROGRESS *rsp, gint current)
 	else
 		GTK_CATCHUP();
 }
+
+void
+gui_progress_add_widget(RS_PROGRESS *rsp, GtkWidget *widget)
+{
+	GtkWidget *alignment;
+
+	alignment = gtk_alignment_new (0.5, 0.5, 1, 1);
+	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 5, 5, 5, 5);
+	gtk_container_add (GTK_CONTAINER (alignment), widget);
+
+	gtk_box_pack_start(GTK_BOX(rsp->vbox), alignment, FALSE, FALSE, 0);
+}
