@@ -48,16 +48,12 @@
 #include "rs-preview-widget.h"
 #include "rs-histogram.h"
 #include "rs-curve.h"
+#include "rs-photo.h"
 
+static void photo_settings_changed(RS_PHOTO *photo, gint mask, RS_BLOB *rs);
+static void photo_spatial_changed(RS_PHOTO *photo, RS_BLOB *rs);
 static RS_SETTINGS *rs_settings_new();
-static void rs_settings_double_free(RS_SETTINGS_DOUBLE *rssd);
-static RS_PHOTO *rs_photo_open_dcraw(const gchar *filename, gboolean half_size);
-void rs_photo_open_dcraw_apply_black_and_shift_half_size(dcraw_data *raw, RS_PHOTO *photo);
-static RS_PHOTO *rs_photo_open_gdk(const gchar *filename, gboolean half_size);
 static GdkPixbuf *rs_thumb_gdk(const gchar *src);
-static void rs_rect_flip(RS_RECT *in, RS_RECT *out, gint w, gint h);
-static void rs_rect_mirror(RS_RECT *in, RS_RECT *out, gint w, gint h);
-static void rs_rect_rotate(RS_RECT *in, RS_RECT *out, gint w, gint h, gint quarterturns);
 
 RS_FILETYPE *filetypes;
 
@@ -107,6 +103,8 @@ rs_init_filetypes(void)
 		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("kdc", FILETYPE_RAW, ".kdc", _("Kodak"),
 		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+	rs_add_filetype("kdc", FILETYPE_RAW, ".dcr", _("Kodak"),
+		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("x3f", FILETYPE_RAW, ".x3f", _("Sigma"),
 		rs_photo_open_dcraw, rs_x3f_load_thumb, NULL, NULL);
 	rs_add_filetype("orf", FILETYPE_RAW, ".orf", "",
@@ -117,6 +115,10 @@ rs_init_filetypes(void)
 		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("dng", FILETYPE_RAW, "dng", _("Adobe Digital negative"),
 		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+	rs_add_filetype("mef", FILETYPE_RAW, "mef", _("Mamiya"),
+		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+	rs_add_filetype("3fr", FILETYPE_RAW, "3fr", _("Hasselblad"),
+		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("jpeg", FILETYPE_JPEG, ".jpg", _("JPEG (Joint Photographic Experts Group)"),
 		rs_photo_open_gdk, rs_thumb_gdk, NULL, rs_photo_save);
 	rs_add_filetype("png", FILETYPE_PNG, ".png", _("PNG (Portable Network Graphics)"),
@@ -128,44 +130,121 @@ rs_init_filetypes(void)
 	return;
 }
 
-void
-rs_update_preview(RS_BLOB *rs)
-{
-	if (!rs->in_use) return;
-
-	rs_preview_widget_update(RS_PREVIEW_WIDGET(rs->preview));
-
-	/* Update histogram */
-	rs_color_transform_set_from_settings(rs->histogram_transform, rs->photo->settings[rs->current_setting], MASK_ALL);
-	rs_histogram_set_color_transform(RS_HISTOGRAM_WIDGET(rs->histogram), rs->histogram_transform);
-
-	rs_curve_draw_histogram(RS_CURVE_WIDGET(rs->settings[rs->current_setting]->curve), rs->histogram_dataset, rs->photo->settings[rs->current_setting]);
-}
-
 void
 rs_reset(RS_BLOB *rs)
 {
-	gboolean in_use = rs->in_use;
 	gint c;
-	rs->in_use = FALSE;
 	for(c=0;c<3;c++)
 		rs_settings_reset(rs->settings[c], MASK_ALL);
-	rs->in_use = in_use;
 	return;
 }
 
 void
 rs_free(RS_BLOB *rs)
 {
-	if (rs->in_use)
+	if (rs->photo)
+		g_object_unref(rs->photo);
+}
+
+static void
+photo_settings_changed(RS_PHOTO *photo, gint mask, RS_BLOB *rs)
+{
+	const gint snapshot = mask>>24;
+	mask &= 0x00ffffff;
+
+	/* Is this really safe? */
+	if (snapshot != rs->current_setting)
+		return;
+
+	if (photo == rs->photo)
 	{
-		if (rs->photo!=NULL)
-			rs_photo_free(rs->photo);
-		rs->photo=NULL;
-		rs->in_use=FALSE;
+		/* Update histogram */
+		rs_color_transform_set_from_settings(rs->histogram_transform, rs->photo->settings[rs->current_setting], mask);
+		rs_histogram_set_color_transform(RS_HISTOGRAM_WIDGET(rs->histogram), rs->histogram_transform);
+
+		/* Update histogram in curve */
+		rs_curve_draw_histogram(RS_CURVE_WIDGET(rs->settings[rs->current_setting]->curve),
+			rs->histogram_dataset,
+			rs->photo->settings[rs->current_setting]);
+
+		/* Update local settings according to photo */
+		rs->mute_signals_to_photo = TRUE;
+		/* FIXME: This 'if' is just a workaround for a bug in rs_photo_apply_settings() */
+		if (mask)
+			rs_settings_double_to_rs_settings(rs->photo->settings[rs->current_setting],
+				rs->settings[rs->current_setting], mask);
+		rs->mute_signals_to_photo = FALSE;
 	}
 }
 
+static void
+photo_spatial_changed(RS_PHOTO *photo, RS_BLOB *rs)
+{
+	if (photo == rs->photo)
+	{
+		/* Update histogram dataset */
+		if (rs->histogram_dataset)
+			rs_image16_free(rs->histogram_dataset);
+
+		rs->histogram_dataset = rs_image16_transform(photo->input, NULL,
+			NULL, NULL, photo->crop, 250, 250,
+			TRUE, -1.0f, photo->angle, photo->orientation, NULL);
+
+		rs_histogram_set_image(RS_HISTOGRAM_WIDGET(rs->histogram), rs->histogram_dataset);
+
+		photo_settings_changed(photo, MASK_ALL, rs);
+	}
+}
+
+void
+rs_set_photo(RS_BLOB *rs, RS_PHOTO *photo)
+{
+	g_assert(rs != NULL);
+
+	/* Unref old photo if any */
+	if (rs->photo)
+		g_object_unref(rs->photo);
+	rs->photo = NULL;
+
+	rs_reset(rs);
+
+	/* Apply settings from photo */
+	rs_settings_double_to_rs_settings(photo->settings[0], rs->settings[0], MASK_ALL);
+	rs_settings_double_to_rs_settings(photo->settings[1], rs->settings[1], MASK_ALL);
+	rs_settings_double_to_rs_settings(photo->settings[2], rs->settings[2], MASK_ALL);
+
+	/* Set photo in preview-widget */
+	rs_preview_widget_set_photo(RS_PREVIEW_WIDGET(rs->preview), photo);
+
+	/* Save photo in blob */
+	rs->photo = photo;
+
+	if (rs->photo)
+	{
+		g_signal_connect(G_OBJECT(rs->photo), "settings-changed", G_CALLBACK(photo_settings_changed), rs);
+		g_signal_connect(G_OBJECT(rs->photo), "spatial-changed", G_CALLBACK(photo_spatial_changed), rs);
+
+		/* Force an update! */
+		photo_spatial_changed(rs->photo, rs);
+	}
+}
+
+void
+rs_set_snapshot(RS_BLOB *rs, gint snapshot)
+{
+	g_assert (rs != NULL);
+	g_assert ((snapshot>=0) && (snapshot<=2));
+
+	rs->current_setting = snapshot;
+
+	/* Switch preview widget to the correct snapshot */
+	rs_preview_widget_set_snapshot(RS_PREVIEW_WIDGET(rs->preview), 0, snapshot);
+
+	/* Force an update */
+	if (rs->photo)
+		photo_settings_changed(rs->photo, MASK_ALL|(snapshot<<24), rs);
+}
+
 void
 rs_settings_to_rs_settings_double(RS_SETTINGS *rs_settings, RS_SETTINGS_DOUBLE *rs_settings_double)
 {
@@ -179,35 +258,53 @@ rs_settings_to_rs_settings_double(RS_SETTINGS *rs_settings, RS_SETTINGS_DOUBLE *
 	rs_settings_double->contrast = GETVAL(rs_settings->contrast);
 	rs_settings_double->warmth = GETVAL(rs_settings->warmth);
 	rs_settings_double->tint = GETVAL(rs_settings->tint);
+	rs_settings_double->sharpen = GETVAL(rs_settings->sharpen);
 	rs_curve_widget_get_knots(RS_CURVE_WIDGET(rs_settings->curve), &rs_settings_double->curve_knots, &rs_settings_double->curve_nknots);
 	return;
 }
 
 void
-rs_settings_double_to_rs_settings(RS_SETTINGS_DOUBLE *rs_settings_double, RS_SETTINGS *rs_settings)
+rs_settings_double_to_rs_settings(RS_SETTINGS_DOUBLE *rs_settings_double, RS_SETTINGS *rs_settings, gint mask)
 {
 	if (rs_settings_double==NULL)
 		return;
 	if (rs_settings==NULL)
 		return;
-	SETVAL(rs_settings->exposure, rs_settings_double->exposure);
+
+	if (mask == 0)
+		return;
+	if (mask & MASK_EXPOSURE)
+		SETVAL(rs_settings->exposure, rs_settings_double->exposure);
+	if (mask & MASK_SATURATION)
 	SETVAL(rs_settings->saturation, rs_settings_double->saturation);
-	SETVAL(rs_settings->hue, rs_settings_double->hue);
-	SETVAL(rs_settings->contrast, rs_settings_double->contrast);
-	SETVAL(rs_settings->warmth, rs_settings_double->warmth);
-	SETVAL(rs_settings->tint, rs_settings_double->tint);
-	rs_curve_widget_reset(RS_CURVE_WIDGET(rs_settings->curve));
-	if (rs_settings_double->curve_nknots>0)
+	if (mask & MASK_HUE)
+		SETVAL(rs_settings->hue, rs_settings_double->hue);
+	if (mask & MASK_CONTRAST)
+		SETVAL(rs_settings->contrast, rs_settings_double->contrast);
+	if (mask & MASK_WARMTH)
+		SETVAL(rs_settings->warmth, rs_settings_double->warmth);
+	if (mask & MASK_TINT)
+		SETVAL(rs_settings->tint, rs_settings_double->tint);
+	if (mask & MASK_SHARPEN)
+		SETVAL(rs_settings->sharpen, rs_settings_double->sharpen);
+	if (mask & MASK_CURVE)
 	{
-		gint i;
-		for(i=0;i<rs_settings_double->curve_nknots;i++)
-			rs_curve_widget_add_knot(RS_CURVE_WIDGET(rs_settings->curve), rs_settings_double->curve_knots[i*2+0],
-				rs_settings_double->curve_knots[i*2+1]);
-	}
-	else
-	{
-		rs_curve_widget_add_knot(RS_CURVE_WIDGET(rs_settings->curve), 0.0f, 0.0f);
-		rs_curve_widget_add_knot(RS_CURVE_WIDGET(rs_settings->curve), 1.0f, 1.0f);
+		gulong handler = g_signal_handler_find(rs_settings->curve, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, update_preview_callback, NULL);
+		g_signal_handler_block(rs_settings->curve, handler);
+		rs_curve_widget_reset(RS_CURVE_WIDGET(rs_settings->curve));
+		if (rs_settings_double->curve_nknots>0)
+		{
+			gint i;
+			for(i=0;i<rs_settings_double->curve_nknots;i++)
+				rs_curve_widget_add_knot(RS_CURVE_WIDGET(rs_settings->curve), rs_settings_double->curve_knots[i*2+0],
+					rs_settings_double->curve_knots[i*2+1]);
+		}
+		else
+		{
+			rs_curve_widget_add_knot(RS_CURVE_WIDGET(rs_settings->curve), 0.0f, 0.0f);
+			rs_curve_widget_add_knot(RS_CURVE_WIDGET(rs_settings->curve), 1.0f, 1.0f);
+		}
+		g_signal_handler_unblock(rs_settings->curve, handler);
 	}
 	return;
 }
@@ -227,6 +324,8 @@ rs_settings_reset(RS_SETTINGS *rss, guint mask)
 		gtk_adjustment_set_value((GtkAdjustment *) rss->warmth, 0.0);
 	if (mask & MASK_TINT)
 		gtk_adjustment_set_value((GtkAdjustment *) rss->tint, 0.0);
+	if (mask & MASK_SHARPEN)
+		gtk_adjustment_set_value((GtkAdjustment *) rss->sharpen, 0.0);
 	if (mask & MASK_CURVE)
 	{
 		rs_curve_widget_reset(RS_CURVE_WIDGET(rss->curve));
@@ -247,6 +346,7 @@ rs_settings_new(void)
 	rss->contrast = gtk_adjustment_new(1.0, 0.0, 3.0, 0.1, 0.5, 0.0);
 	rss->warmth = gtk_adjustment_new(0.0, -2.0, 2.0, 0.1, 0.5, 0.0);
 	rss->tint = gtk_adjustment_new(0.0, -2.0, 2.0, 0.1, 0.5, 0.0);
+	rss->sharpen = gtk_adjustment_new(0.0, 0.0, 10.0, 0.1, 0.5, 0.0);
 	rss->curve = rs_curve_widget_new();
 	return(rss);
 }
@@ -262,13 +362,20 @@ rs_settings_double_new(void)
 	rssd->contrast = 1.0;
 	rssd->warmth = 0.0;
 	rssd->tint = 0.0;
-	rssd->curve_nknots = 0;
-	rssd->curve_knots = NULL;
+	rssd->sharpen = 0.0;
+
+	/* Initialize curve with some sane values */
+	rssd->curve_nknots = 2;
+	rssd->curve_knots = g_new(gfloat, 4);
+	rssd->curve_knots[0] = 0.0;
+	rssd->curve_knots[1] = 0.0;
+	rssd->curve_knots[2] = 1.0;
+	rssd->curve_knots[3] = 1.0;
 	return rssd;
 }
 
 void
-rs_settings_double_copy(RS_SETTINGS_DOUBLE *in, RS_SETTINGS_DOUBLE *out, gint mask)
+rs_settings_double_copy(const RS_SETTINGS_DOUBLE *in, RS_SETTINGS_DOUBLE *out, gint mask)
 {
 	g_assert(in);
 	g_assert(out);
@@ -284,6 +391,8 @@ rs_settings_double_copy(RS_SETTINGS_DOUBLE *in, RS_SETTINGS_DOUBLE *out, gint ma
 		out->warmth = in->warmth;
 	if (mask & MASK_TINT)
 		out->tint = in->tint;
+	if (mask & MASK_SHARPEN)
+		out->sharpen = in->sharpen;
 	if (mask & MASK_CURVE)
 	{
 
@@ -303,7 +412,7 @@ rs_settings_double_copy(RS_SETTINGS_DOUBLE *in, RS_SETTINGS_DOUBLE *out, gint ma
 	return;
 }
 
-static void
+void
 rs_settings_double_free(RS_SETTINGS_DOUBLE *rssd)
 {
 	g_free(rssd);
@@ -369,107 +478,32 @@ rs_metadata_normalize_wb(RS_METADATA *meta)
 	return;
 }
 
-RS_PHOTO *
-rs_photo_new()
-{
-	guint c;
-	RS_PHOTO *photo;
-	photo = g_malloc(sizeof(RS_PHOTO));
-	photo->filename = NULL;
-	if (!photo) return(NULL);
-	photo->input = NULL;
-	ORIENTATION_RESET(photo->orientation);
-	photo->priority = PRIO_U;
-	photo->metadata = rs_metadata_new();
-	for(c=0;c<3;c++)
-		photo->settings[c] = rs_settings_double_new();
-	photo->crop = NULL;
-	photo->angle = 0.0;
-	photo->exported = FALSE;
-	return(photo);
-}
-
-void
-rs_photo_flip(RS_PHOTO *photo)
-{
-	if (photo->crop)
-	{
-		gint w,h;
-		rs_image16_transform_getwh(photo->input, NULL, photo->angle, photo->orientation, &w, &h);
-		rs_rect_flip(photo->crop, photo->crop, w, h);
-	}
-	ORIENTATION_FLIP(photo->orientation);
-}
-
-void
-rs_photo_mirror(RS_PHOTO *photo)
-{
-	if (photo->crop)
-	{
-		gint w,h;
-		rs_image16_transform_getwh(photo->input, NULL, photo->angle, photo->orientation, &w, &h);
-		rs_rect_mirror(photo->crop, photo->crop, w, h);
-	}
-	ORIENTATION_MIRROR(photo->orientation);
-}
-
-void
-rs_photo_rotate(RS_PHOTO *photo, gint quarterturns, gdouble angle)
-{
-	gint n;
-	photo->angle += angle;
-
-	if (photo->crop)
-	{
-		gint w,h;
-		rs_image16_transform_getwh(photo->input, NULL, photo->angle, photo->orientation, &w, &h);
-		rs_rect_rotate(photo->crop, photo->crop, w, h, quarterturns);
-	}
-
-	for(n=0;n<quarterturns;n++)
-		ORIENTATION_90(photo->orientation);
-
-	return;
-
-}
-
-void
-rs_photo_set_crop(RS_PHOTO *photo, RS_RECT *crop)
-{
-	if (photo->crop)
-		g_free(photo->crop);
-	photo->crop = NULL;
-
-	if (crop)
-	{
-		photo->crop = g_new(RS_RECT, 1);
-		*photo->crop = *crop;
-	}
-}
-
-RS_RECT *
-rs_photo_get_crop(RS_PHOTO *photo)
-{
-	return photo->crop;
-}
-
 gboolean
 rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width, gint height, gboolean keep_aspect, gdouble scale, gint snapshot, RS_CMS *cms)
 {
 	GdkPixbuf *pixbuf;
 	RS_IMAGE16 *rsi;
-	RS_IMAGE8 *image8;
+	RS_IMAGE16 *sharpened;
 	RS_IMAGE16 *image16;
 	gint quality = 100;
 	gboolean uncompressed_tiff = FALSE;
 	RS_COLOR_TRANSFORM *rct;
 	void *transform = NULL;
 
+	g_assert(RS_IS_PHOTO(photo));
+	g_assert(filename != NULL);
+
+	rs_image16_demosaic(photo->input, RS_DEMOSAIC_PPG);
+
+	sharpened = rs_image16_sharpen(photo->input, NULL, photo->settings[snapshot]->sharpen, NULL);
+
 	/* transform and crop */
-	rsi = rs_image16_transform(photo->input, NULL,
+	rsi = rs_image16_transform(sharpened, NULL,
 			NULL, NULL, photo->crop, width, height, keep_aspect, scale,
 			photo->angle, photo->orientation, NULL);
 
+	rs_image16_unref(sharpened);
+
 	if (cms)
 		transform = rs_cms_get_transform(cms, TRANSFORM_EXPORT);
 
@@ -478,14 +512,15 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 	rs_color_transform_set_from_settings(rct, photo->settings[snapshot], MASK_ALL);
 	rs_color_transform_set_output_format(rct, 8);
 	rs_color_transform_set_cms_transform(rct, transform);
+	rs_color_transform_set_adobe_matrix(rct, &photo->metadata->adobe_coeff);
 
 	/* actually save */
 	switch (filetype)
 	{
 		case FILETYPE_JPEG:
-			image8 = rs_image8_new(rsi->w, rsi->h, 3, 3);
+			pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, rsi->w, rsi->h);
 			rct->transform(rct, rsi->w, rsi->h, rsi->pixels,
-				rsi->rowstride, image8->pixels, image8->rowstride);
+				rsi->rowstride, gdk_pixbuf_get_pixels(pixbuf), gdk_pixbuf_get_rowstride(pixbuf));
 
 			rs_conf_get_integer(CONF_EXPORT_JPEG_QUALITY, &quality);
 			if (quality > 100)
@@ -493,8 +528,8 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 			else if (quality < 0)
 				quality = 0;
 
-			rs_jpeg_save(image8, filename, quality, rs_cms_get_profile_filename(cms, PROFILE_EXPORT));
-			rs_image8_free(image8);
+			rs_jpeg_save(pixbuf, filename, quality, rs_cms_get_profile_filename(cms, PROFILE_EXPORT));
+			g_object_unref(pixbuf);
 			break;
 		case FILETYPE_PNG:
 			pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, rsi->w, rsi->h);
@@ -504,12 +539,12 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 			g_object_unref(pixbuf);
 			break;
 		case FILETYPE_TIFF8:
+			pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, rsi->w, rsi->h);
 			rs_conf_get_boolean(CONF_EXPORT_TIFF_UNCOMPRESSED, &uncompressed_tiff);
-			image8 = rs_image8_new(rsi->w, rsi->h, 3, 3);
 			rct->transform(rct, rsi->w, rsi->h, rsi->pixels,
-				rsi->rowstride, image8->pixels, image8->rowstride);
-			rs_tiff8_save(image8, filename, rs_cms_get_profile_filename(cms, PROFILE_EXPORT), uncompressed_tiff);
-			rs_image8_free(image8);
+				rsi->rowstride, gdk_pixbuf_get_pixels(pixbuf), gdk_pixbuf_get_rowstride(pixbuf));
+			rs_tiff8_save(pixbuf, filename, rs_cms_get_profile_filename(cms, PROFILE_EXPORT), uncompressed_tiff);
+			g_object_unref(pixbuf);
 			break;
 		case FILETYPE_TIFF16:
 			rs_conf_get_boolean(CONF_EXPORT_TIFF_UNCOMPRESSED, &uncompressed_tiff);
@@ -534,33 +569,6 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 	return(TRUE);
 }
 
-void
-rs_photo_free(RS_PHOTO *photo)
-{
-	guint c;
-	if (!photo) return;
-	g_free(photo->filename);
-	if (photo->metadata)
-	{
-		rs_metadata_free(photo->metadata);
-		photo->metadata = NULL;
-	}
-	if (photo->input)
-	{
-		rs_image16_free(photo->input);
-		photo->input = NULL;
-	}
-	for(c=0;c<3;c++)
-		rs_settings_double_free(photo->settings[c]);
-	if (photo->crop)
-	{
-		g_free(photo->crop);
-		photo->crop = NULL;
-	}
-	g_free(photo);
-	return;
-}
-
 RS_BLOB *
 rs_new(void)
 {
@@ -570,7 +578,7 @@ rs_new(void)
 	rs->histogram_dataset = NULL;
 	rs->histogram_transform = rs_color_transform_new();
 	rs->settings_buffer = NULL;
-	rs->in_use = FALSE;
+	rs->mute_signals_to_photo = FALSE;
 	rs->photo = NULL;
 	rs->queue = rs_batch_new_queue();
 	rs->current_setting = 0;
@@ -579,204 +587,11 @@ rs_new(void)
 	return(rs);
 }
 
-void
-rs_photo_close(RS_PHOTO *photo)
-{
-	if (!photo) return;
-	rs_cache_save(photo);
-	return;
-}
-
-static RS_PHOTO *
-rs_photo_open_dcraw(const gchar *filename, gboolean half_size)
-{
-	dcraw_data *raw;
-	RS_PHOTO *photo=NULL;
-
-	raw = (dcraw_data *) g_malloc(sizeof(dcraw_data));
-	if (!dcraw_open(raw, (char *) filename))
-	{
-		dcraw_load_raw(raw);
-		photo = rs_photo_new(NULL);
-
-		if (half_size)
-		{
-			photo->input = rs_image16_new(raw->raw.width, raw->raw.height, raw->raw.colors, 4);
-			rs_photo_open_dcraw_apply_black_and_shift_half_size(raw, photo);
-		}
-		else
-		{
-			photo->input = rs_image16_new(raw->raw.width*2, raw->raw.height*2, raw->raw.colors, 4);
-			rs_photo_open_dcraw_apply_black_and_shift(raw, photo);
-		}
-
-		photo->input->filters = raw->filters;
-		photo->input->fourColorFilters = raw->fourColorFilters;
-		photo->filename = g_strdup(filename);
-		dcraw_close(raw);
-	}
-	g_free(raw);
-	return(photo);
-}
-
-void
-rs_photo_open_dcraw_apply_black_and_shift_half_size(dcraw_data *raw, RS_PHOTO *photo)
-{
-	gushort *dst, *src;
-	gint row, col;
-	gint64 shift = (gint64) (16.0-log((gdouble) raw->rgbMax)/log(2.0)+0.5);
-
-	for(row=0;row<(raw->raw.height);row++)
-	{
-		src = (gushort *) raw->raw.image + row * raw->raw.width * 4;
-		dst = GET_PIXEL(photo->input, 0, row);
-		col = raw->raw.width;
-		while(col--)
-		{
-			register gint r, g, b, g2;
-			r  = *src++ - raw->black;
-			g  = *src++ - raw->black;
-			b  = *src++ - raw->black;
-			g2 = *src++ - raw->black;
-			r  = MAX(0, r);
-			g  = MAX(0, g);
-			b  = MAX(0, b);
-			g2 = MAX(0, g2);
-			*dst++ = (gushort)( r<<shift);
-			*dst++ = (gushort)( g<<shift);
-			*dst++ = (gushort)( b<<shift);
-			*dst++ = (gushort)(g2<<shift);
-		}
-	}
-}
-
-/* Function pointer. Initiliazed by arch binder */
-void
-(*rs_photo_open_dcraw_apply_black_and_shift)(dcraw_data *raw, RS_PHOTO *photo);
-
-void
-rs_photo_open_dcraw_apply_black_and_shift_c(dcraw_data *raw, RS_PHOTO *photo)
-{
-	gushort *dst1, *dst2, *src;
-	gint row, col;
-	gint64 shift = (gint64) (16.0-log((gdouble) raw->rgbMax)/log(2.0)+0.5);
-
-	for(row=0;row<(raw->raw.height*2);row+=2)
-	{
-		src = (gushort *) raw->raw.image + row/2 * raw->raw.width * 4;
-		dst1 = GET_PIXEL(photo->input, 0, row);
-		dst2 = GET_PIXEL(photo->input, 0, row+1);
-		col = raw->raw.width;
-		while(col--)
-		{
-			register gint r, g, b, g2;
-			r  = *src++ - raw->black;
-			g  = *src++ - raw->black;
-			b  = *src++ - raw->black;
-			g2 = *src++ - raw->black;
-			r  = MAX(0, r);
-			g  = MAX(0, g);
-			b  = MAX(0, b);
-			g2 = MAX(0, g2);
-			*dst1++ = (gushort)( r<<shift);
-			*dst1++ = (gushort)( g<<shift);
-			*dst1++ = (gushort)( b<<shift);
-			*dst1++ = (gushort)(g2<<shift);
-			*dst1++ = (gushort)( r<<shift);
-			*dst1++ = (gushort)( g<<shift);
-			*dst1++ = (gushort)( b<<shift);
-			*dst1++ = (gushort)(g2<<shift);
-			*dst2++ = (gushort)( r<<shift);
-			*dst2++ = (gushort)( g<<shift);
-			*dst2++ = (gushort)( b<<shift);
-			*dst2++ = (gushort)(g2<<shift);
-			*dst2++ = (gushort)( r<<shift);
-			*dst2++ = (gushort)( g<<shift);
-			*dst2++ = (gushort)( b<<shift);
-			*dst2++ = (gushort)(g2<<shift);
-		}
-	}
-}
-
-#if defined (__i386__) || defined (__x86_64__)
-void
-rs_photo_open_dcraw_apply_black_and_shift_mmx(dcraw_data *raw, RS_PHOTO *photo)
-{
-	char b[8];
-	volatile gushort *sub = (gushort *) b;
-	void *srcoffset;
-	void *destoffset;
-	guint x;
-	guint y;
-	gushort *src = (gushort*)raw->raw.image;
-	volatile gint64 shift = (gint64) (16.0-log((gdouble) raw->rgbMax)/log(2.0)+0.5);
-
-	sub[0] = raw->black;
-	sub[1] = raw->black;
-	sub[2] = raw->black;
-	sub[3] = raw->black;
-
-	for (y=0; y<(raw->raw.height*2); y++)
-	{
-		destoffset = (void*) (photo->input->pixels + y*photo->input->rowstride);
-		srcoffset = (void*) (src + y/2 * raw->raw.width * photo->input->pixelsize);
-		x = raw->raw.width;
-		asm volatile (
-			"mov %3, %%"REG_a"\n\t" /* copy x to %eax */
-			"movq (%2), %%mm7\n\t" /* put black in %mm7 */
-			"movq (%4), %%mm6\n\t" /* put shift in %mm6 */
-			".p2align 4,,15\n"
-			"load_raw_inner_loop:\n\t"
-			"movq (%1), %%mm0\n\t" /* load source */
-			"movq 8(%1), %%mm1\n\t"
-			"movq 16(%1), %%mm2\n\t"
-			"movq 24(%1), %%mm3\n\t"
-			"psubusw %%mm7, %%mm0\n\t" /* subtract black */
-			"psubusw %%mm7, %%mm1\n\t"
-			"psubusw %%mm7, %%mm2\n\t"
-			"psubusw %%mm7, %%mm3\n\t"
-			"psllw %%mm6, %%mm0\n\t" /* bitshift */
-			"psllw %%mm6, %%mm1\n\t"
-			"psllw %%mm6, %%mm2\n\t"
-			"psllw %%mm6, %%mm3\n\t"
-			"movq %%mm0, (%0)\n\t" /* write destination (twice) */
-			"movq %%mm0, 8(%0)\n\t"
-			"movq %%mm1, 16(%0)\n\t"
-			"movq %%mm1, 24(%0)\n\t"
-			"movq %%mm2, 32(%0)\n\t"
-			"movq %%mm2, 40(%0)\n\t"
-			"movq %%mm3, 48(%0)\n\t"
-			"movq %%mm3, 56(%0)\n\t"
-			"sub $4, %%"REG_a"\n\t"
-			"add $64, %0\n\t"
-			"add $32, %1\n\t"
-			"cmp $3, %%"REG_a"\n\t"
-			"jg load_raw_inner_loop\n\t"
-			"cmp $1, %%"REG_a"\n\t"
-			"jb load_raw_inner_done\n\t"
-			".p2align 4,,15\n"
-			"load_raw_leftover:\n\t"
-			"movq (%1), %%mm0\n\t" /* leftover pixels */
-			"psubusw %%mm7, %%mm0\n\t"
-			"psllw %%mm6, %%mm0\n\t"
-			"movq %%mm0, (%0)\n\t"
-			"sub $1, %%"REG_a"\n\t"
-			"cmp $0, %%"REG_a"\n\t"
-			"jg load_raw_leftover\n\t"
-			"load_raw_inner_done:\n\t"
-			"emms\n\t" /* clean up */
-			: "+r" (destoffset), "+r" (srcoffset)
-			: "r" (sub), "r" ((gulong)x), "r" (&shift)
-			: "%"REG_a
-			);
-	}
-}
-#endif
-
 RS_FILETYPE *
 rs_filetype_get(const gchar *filename, gboolean load)
 {
 	RS_FILETYPE *filetype = filetypes;
+	RS_FILETYPE *ret = NULL;
 	gchar *iname;
 	gint n;
 	gboolean load_gdk = FALSE;
@@ -790,64 +605,16 @@ rs_filetype_get(const gchar *filename, gboolean load)
 			if ((!load) || (filetype->load))
 			{
 				if (filetype->filetype == FILETYPE_RAW)
-					return(filetype);
+					ret = filetype;
 				else if ((filetype->filetype != FILETYPE_RAW) && (load_gdk))
-					return(filetype);
+					ret = filetype;
 				break;
 			}
 		}
 		filetype = filetype->next;
 	}
 	g_free(iname);
-	return(NULL);
-}
-
-static RS_PHOTO *
-rs_photo_open_gdk(const gchar *filename, gboolean half_size)
-{
-	RS_PHOTO *photo=NULL;
-	GdkPixbuf *pixbuf;
-	guchar *pixels;
-	gint rowstride;
-	gint width, height;
-	gint row,col,n,res, src, dest;
-	gdouble nd;
-	gushort gammatable[256];
-	gint alpha=0;
-	if ((pixbuf = gdk_pixbuf_new_from_file(filename, NULL)))
-	{
-		photo = rs_photo_new();
-		for(n=0;n<256;n++)
-		{
-			nd = ((gdouble) n) / 255.0;
-			res = (gint) (pow(nd, GAMMA) * 65535.0);
-			_CLAMP65535(res);
-			gammatable[n] = res;
-		}
-		rowstride = gdk_pixbuf_get_rowstride(pixbuf);
-		pixels = gdk_pixbuf_get_pixels(pixbuf);
-		width = gdk_pixbuf_get_width(pixbuf);
-		height = gdk_pixbuf_get_height(pixbuf);
-		if (gdk_pixbuf_get_has_alpha(pixbuf))
-			alpha = 1;
-		photo->input = rs_image16_new(width, height, 3, 4);
-		for(row=0;row<photo->input->h;row++)
-		{
-			dest = row * photo->input->rowstride;
-			src = row * rowstride;
-			for(col=0;col<photo->input->w;col++)
-			{
-				photo->input->pixels[dest++] = gammatable[pixels[src++]];
-				photo->input->pixels[dest++] = gammatable[pixels[src++]];
-				photo->input->pixels[dest++] = gammatable[pixels[src++]];
-				photo->input->pixels[dest++] = gammatable[pixels[src-2]];
-				src+=alpha;
-			}
-		}
-		g_object_unref(pixbuf);
-		photo->filename = g_strdup(filename);
-	}
-	return(photo);
+	return ret;
 }
 
 gchar *
@@ -1000,91 +767,6 @@ rs_white_black_point(RS_BLOB *rs)
 	}
 }
 
-void
-rs_set_wb_auto(RS_BLOB *rs)
-{
-	gint row, col, x, y, c, val;
-	gint sum[8];
-	gdouble pre_mul[4];
-	gdouble dsum[8];
-
-	if(unlikely(!rs->photo)) return;
-
-	for (c=0; c < 8; c++)
-		dsum[c] = 0.0;
-
-	for (row=0; row < rs->photo->input->h-7; row += 8)
-		for (col=0; col < rs->photo->input->w-7; col += 8)
-		{
-			memset (sum, 0, sizeof sum);
-			for (y=row; y < row+8; y++)
-				for (x=col; x < col+8; x++)
-					for(c=0;c<4;c++)
-					{
-						val = rs->photo->input->pixels[y*rs->photo->input->rowstride+x*4+c];
-						if (!val) continue;
-						if (val > 65100)
-							goto skip_block; /* I'm sorry mom */
-						sum[c] += val;
-						sum[c+4]++;
-					}
-			for (c=0; c < 8; c++)
-				dsum[c] += sum[c];
-skip_block:
-							continue;
-		}
-	for(c=0;c<4;c++)
-		if (dsum[c])
-			pre_mul[c] = dsum[c+4] / dsum[c];
-	rs_set_wb_from_mul(rs, pre_mul);
-	return;
-}
-
-void
-rs_set_wb_from_color(RS_BLOB *rs, gdouble r, gdouble g, gdouble b)
-{
-	gdouble warmth, tint;
-	warmth = (b-r)/(r+b); /* r*(1+warmth) = b*(1-warmth) */
-	tint = -g/(r+r*warmth)+2.0; /* magic */
-	rs_set_wb(rs, warmth, tint);
-	return;
-}
-
-void
-rs_set_wb_from_mul(RS_BLOB *rs, gdouble *mul)
-{
-	int c;
-	gdouble max=0.0, warmth, tint;
-
-	for (c=0; c < 4; c++)
-		if (max < mul[c])
-			max = mul[c];
-
-	for(c=0;c<4;c++)
-		mul[c] /= max;
-
-	mul[R] *= (1.0/mul[G]);
-	mul[B] *= (1.0/mul[G]);
-	mul[G] = 1.0;
-	mul[G2] = 1.0;
-
-	tint = (mul[B] + mul[R] - 4.0)/-2.0;
-	warmth = (mul[R]/(2.0-tint))-1.0;
-	rs_set_wb(rs, warmth, tint);
-	return;
-}
-
-void
-rs_set_wb(RS_BLOB *rs, gfloat warmth, gfloat tint)
-{
-	gboolean in_use = rs->in_use;
-	rs->in_use = FALSE;
-	SETVAL(rs->settings[rs->current_setting]->warmth, warmth);
-	rs->in_use = in_use;
-	SETVAL(rs->settings[rs->current_setting]->tint, tint);
-	return;
-}
-
 void
 rs_apply_settings_from_double(RS_SETTINGS *rss, RS_SETTINGS_DOUBLE *rsd, gint mask)
 {
@@ -1121,26 +803,6 @@ rs_apply_settings_from_double(RS_SETTINGS *rss, RS_SETTINGS_DOUBLE *rsd, gint ma
 	return;
 }
 
-void
-rs_rect_scale(RS_RECT *in, RS_RECT *out, gdouble scale)
-{
-	out->x1 = (gint) (((gdouble) in->x1)*scale);
-	out->x2 = (gint) (((gdouble) in->x2)*scale);
-	out->y1 = (gint) (((gdouble) in->y1)*scale);
-	out->y2 = (gint) (((gdouble) in->y2)*scale);
-	return;
-}
-
-void
-rs_rect_union(RS_RECT *a, RS_RECT *b, RS_RECT *destination)
-{
-	destination->x1 = (a->x1 > b->x1) ? a->x1 : b->x1;
-	destination->y1 = (a->y1 > b->y1) ? a->y1 : b->y1;
-	destination->x2 = (a->x2 < b->x2) ? a->x2 : b->x2;
-	destination->y2 = (a->y2 < b->y2) ? a->y2 : b->y2;
-	return;
-}
-
 void
 rs_rect_normalize(RS_RECT *in, RS_RECT *out)
 {
@@ -1172,7 +834,7 @@ rs_rect_normalize(RS_RECT *in, RS_RECT *out)
 	out->y2 = y2;
 }
 
-static void
+void
 rs_rect_flip(RS_RECT *in, RS_RECT *out, gint w, gint h)
 {
 	gint x1,y1;
@@ -1192,7 +854,7 @@ rs_rect_flip(RS_RECT *in, RS_RECT *out, gint w, gint h)
 	return;
 }
 
-static void
+void
 rs_rect_mirror(RS_RECT *in, RS_RECT *out, gint w, gint h)
 {
 	gint x1,y1;
@@ -1212,7 +874,7 @@ rs_rect_mirror(RS_RECT *in, RS_RECT *out, gint w, gint h)
 	return;
 }
 
-static void
+void
 rs_rect_rotate(RS_RECT *in, RS_RECT *out, gint w, gint h, gint quarterturns)
 {
 	gint x1,y1;
@@ -1254,31 +916,118 @@ rs_rect_rotate(RS_RECT *in, RS_RECT *out, gint w, gint h, gint quarterturns)
 	return;
 }
 
+inline void
+rs_rect_from_gdkrectangle(GdkRectangle *in, RS_RECT *out)
+{
+	out->x1 = in->x;
+	out->y1 = in->y;
+	out->x2 = in->x+in->width;
+	out->y2 = in->y+in->height;
+}
+
 void
 check_install()
 {
 #define TEST_FILE_ACCESS(path) do { if (g_access(path, R_OK)!=0) g_debug("Cannot access %s\n", path);} while (0)
-	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority1.png");
-	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority2.png");
-	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority3.png");
-	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_deleted.png");
-	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_exported.png");
-	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio.png");
-	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_flip.png");
-	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_mirror.png");
-	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_90.png");
-	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_180.png");
-	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_270.png");
-
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/icons/" PACKAGE ".png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/overlay_priority1.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/overlay_priority2.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/overlay_priority3.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/overlay_deleted.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/overlay_exported.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/transform_flip.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/transform_mirror.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/transform_90.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/transform_180.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/transform_270.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/" PACKAGE "/ui.xml");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/" PACKAGE "/rawstudio.gtkrc");
 #undef TEST_FILE_ACCESS
 }
 
+gboolean
+rs_has_gimp(gint major, gint minor, gint micro) {
+	FILE *fp;
+	char line[128];
+	int _major, _minor, _micro;
+	gboolean retval = FALSE;
+
+	fp = popen("gimp -v","r");
+	fgets( line, sizeof line, fp);
+	pclose(fp);
+
+#if GLIB_CHECK_VERSION(2,14,0)
+	GRegex *regex;
+	gchar **tokens;
+	
+	regex = g_regex_new(".*([0-9])\x2E([0-9]+)\x2E([0-9]+).*", 0, 0, NULL);
+	tokens = g_regex_split(regex, line, 0);
+	g_regex_unref(regex);
+
+	if (tokens[1])
+		_major = atoi(tokens[1]);
+	else
+	{
+		g_strfreev(tokens);
+		return FALSE;
+	}
+
+	if (_major > major) {
+		retval = TRUE;
+	} else if (_major == major) {
+
+		if (tokens[2])
+			_minor = atoi(tokens[2]);
+		else
+		{
+			g_strfreev(tokens);
+			return FALSE;
+		}
+
+		if (_minor > minor) {
+			retval = TRUE;
+		} else if (_minor == minor) {
+	
+			if (tokens[3])
+				_micro = atoi(tokens[3]);
+			else
+			{
+				g_strfreev(tokens);
+				return FALSE;
+			}
+
+			if (_micro >= micro) {
+				retval = TRUE;
+			}
+		}
+	}
+	g_strfreev(tokens);
+#else
+	sscanf(line,"GNU Image Manipulation Program version %d.%d.%d", &_major, &_minor, &_micro);
+
+	if (_major > major) {
+		retval = TRUE;
+	} else if (_major == major) {
+		if (_minor > minor) {
+			retval = TRUE;
+		} else if (_minor == minor) {
+			if (_micro >= micro) {
+				retval = TRUE;
+			}
+		}
+	}
+#endif
+
+	return retval;
+}
+
 int
 main(int argc, char **argv)
 {
 	RS_BLOB *rs;
 	int optimized = 1;
 	int opt;
+	gboolean use_system_theme = DEFAULT_CONF_USE_SYSTEM_THEME;
 
 	while ((opt = getopt(argc, argv, "n")) != -1) {
 		switch (opt) {
@@ -1304,6 +1053,12 @@ main(int argc, char **argv)
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
 #endif
+
+	/* Switch to rawstudio theme before any drawing if needed */
+	rs_conf_get_boolean_with_default(CONF_USE_SYSTEM_THEME, &use_system_theme, DEFAULT_CONF_USE_SYSTEM_THEME);
+	if (!use_system_theme)
+		gui_select_theme(RAWSTUDIO_THEME);
+
 	rs_init_filetypes();
 	gtk_init(&argc, &argv);
 	check_install();
@@ -1320,7 +1075,7 @@ gboolean
 rs_shutdown(GtkWidget *dummy1, GdkEvent *dummy2, RS_BLOB *rs)
 {
 	rs_photo_close(rs->photo);
-	rs_conf_set_integer(CONF_LAST_PRIORITY_PAGE, gtk_notebook_get_current_page(GTK_NOTEBOOK(rs->store)));
+	rs_conf_set_integer(CONF_LAST_PRIORITY_PAGE, rs_store_get_current_page(rs->store));
 	gtk_main_quit();
 	return(TRUE);
 }
