@@ -60,7 +60,8 @@ enum {
 	MASK_WARMTH = 16,
 	MASK_TINT = 32,
 	MASK_CURVE = 64,
-	MASK_ALL = 127
+	MASK_SHARPEN = 128,
+	MASK_ALL = 0x00ffffff,
 };
 
 #define MASK_WB (MASK_WARMTH|MASK_TINT)
@@ -120,17 +121,6 @@ typedef struct {int coeff[3][3]; } RS_MATRIX3Int;
 typedef struct {double coeff[4][4]; } RS_MATRIX4;
 typedef struct {int coeff[4][4]; } RS_MATRIX4Int;
 
-typedef struct {
-	gint w;
-	gint h;
-	gint rowstride;
-	guint channels;
-	guint pixelsize; /* the size of a pixel in CHARS */
-	guint orientation;
-	guchar *pixels;
-	gint reference_count;
-} RS_IMAGE8;
-
 typedef struct _rs_image16 {
 	GObject parent;
 	gint w;
@@ -161,6 +151,7 @@ typedef struct {
 	GtkObject *contrast;
 	GtkObject *warmth;
 	GtkObject *tint;
+	GtkObject *sharpen;
 	GtkWidget *curve;
 } RS_SETTINGS;
 
@@ -171,6 +162,7 @@ typedef struct {
 	gdouble contrast;
 	gdouble warmth;
 	gdouble tint;
+	gdouble sharpen;
 	guint curve_nknots;
 	gfloat *curve_knots;
 } RS_SETTINGS_DOUBLE;
@@ -202,6 +194,7 @@ typedef struct _metadata {
 } RS_METADATA;
 
 typedef struct _photo {
+	GObject parent;
 	gchar *filename;
 	RS_IMAGE16 *input;
 	RS_SETTINGS_DOUBLE *settings[3];
@@ -211,10 +204,11 @@ typedef struct _photo {
 	RS_RECT *crop;
 	gdouble angle;
 	gboolean exported;
+	gboolean dispose_has_run;
 } RS_PHOTO;
 
 typedef struct {
-	gboolean in_use;
+	gboolean mute_signals_to_photo;
 	RS_PHOTO *photo;
 	RS_SETTINGS_DOUBLE *settings_buffer;
 	RS_SETTINGS *settings[3];
@@ -225,6 +219,11 @@ typedef struct {
 	RS_QUEUE *queue;
 	RS_CMS *cms;
 	RSStore *store;
+
+	/* These should be moved to a future RS_WINDOW */
+	GtkWidget *window;
+	GtkWidget *iconbox;
+	GtkWidget *toolbox;
 	GtkWidget *preview;
 } RS_BLOB;
 
@@ -250,59 +249,41 @@ typedef struct _rs_filetype {
 
 void rs_local_cachedir(gboolean new_value);
 void rs_load_gdk(gboolean new_value);
-void rs_update_preview(RS_BLOB *rs);
 void rs_reset(RS_BLOB *rs);
 void rs_settings_to_rs_settings_double(RS_SETTINGS *rs_settings, RS_SETTINGS_DOUBLE *rs_settings_double);
-void rs_settings_double_to_rs_settings(RS_SETTINGS_DOUBLE *rs_settings_double, RS_SETTINGS *rs_settings);
+void rs_settings_double_to_rs_settings(RS_SETTINGS_DOUBLE *rs_settings_double, RS_SETTINGS *rs_settings, gint mask);
 void rs_settings_reset(RS_SETTINGS *rss, guint mask);
-RS_PHOTO *rs_photo_new();
-void rs_photo_free(RS_PHOTO *photo);
 gboolean rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype,
 	gint width, gint height, gboolean keep_aspect, gdouble scale, gint snapshot, RS_CMS *cms);
 RS_SETTINGS_DOUBLE *rs_settings_double_new(void);
-void rs_settings_double_copy(RS_SETTINGS_DOUBLE *in, RS_SETTINGS_DOUBLE *out, gint mask);
+void rs_settings_double_copy(const RS_SETTINGS_DOUBLE *in, RS_SETTINGS_DOUBLE *out, gint mask);
+void rs_settings_double_free(RS_SETTINGS_DOUBLE *rssd);
 RS_METADATA *rs_metadata_new();
 void rs_metadata_free(RS_METADATA *metadata);
 void rs_metadata_normalize_wb(RS_METADATA *meta);
-void rs_photo_flip(RS_PHOTO *photo);
-void rs_photo_mirror(RS_PHOTO *photo);
-void rs_photo_rotate(RS_PHOTO *photo, gint quarterturns, gdouble angle);
-void rs_photo_set_crop(RS_PHOTO *photo, RS_RECT *crop);
-RS_RECT *rs_photo_get_crop(RS_PHOTO *photo);
 RS_BLOB *rs_new();
 void rs_free(RS_BLOB *rs);
-void rs_photo_close(RS_PHOTO *photo);
+void rs_set_photo(RS_BLOB *rs, RS_PHOTO *photo);
+void rs_set_snapshot(RS_BLOB *rs, gint snapshot);
 RS_FILETYPE *rs_filetype_get(const gchar *filename, gboolean load);
 gchar *rs_confdir_get();
 gchar *rs_dotdir_get(const gchar *filename);
 gchar *rs_thumb_get_name(const gchar *src);
 void rs_white_black_point(RS_BLOB *rs);
-void rs_set_wb_auto(RS_BLOB *rs);
-void rs_set_wb_from_color(RS_BLOB *rs, gdouble r, gdouble g, gdouble b);
-void rs_set_wb_from_mul(RS_BLOB *rs, gdouble *mul);
-void rs_set_wb(RS_BLOB *rs, gfloat warmth, gfloat tint);
 void rs_render_pixel_to_srgb(RS_BLOB *rs, gint x, gint y, guchar *dest);
 void rs_apply_settings_from_double(RS_SETTINGS *rss, RS_SETTINGS_DOUBLE *rsd, gint mask);
-void rs_rect_scale(RS_RECT *in, RS_RECT *out, gdouble scale);
-void rs_rect_union(RS_RECT *a, RS_RECT *b, RS_RECT *destination);
 void rs_rect_normalize(RS_RECT *in, RS_RECT *out);
 gboolean rs_shutdown(GtkWidget *dummy1, GdkEvent *dummy2, RS_BLOB *rs);
+gboolean rs_has_gimp(gint major, gint minor, gint micro);
+void rs_rect_flip(RS_RECT *in, RS_RECT *out, gint w, gint h);
+void rs_rect_mirror(RS_RECT *in, RS_RECT *out, gint w, gint h);
+void rs_rect_rotate(RS_RECT *in, RS_RECT *out, gint w, gint h, gint quarterturns);
+extern inline void rs_rect_from_gdkrectangle(GdkRectangle *in, RS_RECT *out);
 #if !GLIB_CHECK_VERSION(2,8,0)
 int g_mkdir_with_parents (const gchar *pathname, int mode);
 int g_access (const gchar *filename, int mode);
 #endif
 
-extern void
-(*rs_photo_open_dcraw_apply_black_and_shift)(dcraw_data *raw, RS_PHOTO *photo) __rs_optimized;
-
-extern void
-rs_photo_open_dcraw_apply_black_and_shift_c(dcraw_data *raw, RS_PHOTO *photo);
-
-#if defined (__i386__) || defined (__x86_64__)
-extern void
-rs_photo_open_dcraw_apply_black_and_shift_mmx(dcraw_data *raw, RS_PHOTO *photo);
-#endif
-
 /* Contains a list of supported filetypes */
 extern RS_FILETYPE *filetypes;
 
