@@ -69,6 +69,7 @@ batch_queue_save(RS_QUEUE *queue)
 				xmlTextWriterWriteFormatElement(writer, BAD_CAST "filename", "%s", filename);
 				xmlTextWriterWriteFormatElement(writer, BAD_CAST "snapshot", "%d", setting_id);
 			xmlTextWriterEndElement(writer);
+			g_free(filename);
 		} while(gtk_tree_model_iter_next(queue->list, &iter));
 
 	xmlTextWriterEndDocument(writer);
@@ -257,21 +258,24 @@ rs_batch_add_to_queue(RS_QUEUE *queue, const gchar *filename, const gint setting
 				pixbuf = missing_thumb;
 				g_object_ref (pixbuf);
 			}
-		}
+			g_object_unref(missing_thumb);
 
-		if (!batch_exists_in_queue(queue, filename, setting_id))
-		{
-			GtkTreeIter iter;
+			if (!batch_exists_in_queue(queue, filename, setting_id))
+			{
+				GtkTreeIter iter;
 
-			gtk_list_store_append (GTK_LIST_STORE(queue->list), &iter);
- 			gtk_list_store_set (GTK_LIST_STORE(queue->list), &iter,
-				RS_QUEUE_ELEMENT_FILENAME, filename,
-				RS_QUEUE_ELEMENT_FILENAME_SHORT, filename_short,
-				RS_QUEUE_ELEMENT_SETTING_ID, setting_id,
-				RS_QUEUE_ELEMENT_SETTING_ID_ABC, setting_id_abc,
-				RS_QUEUE_ELEMENT_THUMBNAIL, pixbuf,
-				-1);
-			ret = TRUE;
+				gtk_list_store_append (GTK_LIST_STORE(queue->list), &iter);
+				gtk_list_store_set (GTK_LIST_STORE(queue->list), &iter,
+					RS_QUEUE_ELEMENT_FILENAME, filename,
+					RS_QUEUE_ELEMENT_FILENAME_SHORT, filename_short,
+					RS_QUEUE_ELEMENT_SETTING_ID, setting_id,
+					RS_QUEUE_ELEMENT_SETTING_ID_ABC, setting_id_abc,
+					RS_QUEUE_ELEMENT_THUMBNAIL, pixbuf,
+					-1);
+				ret = TRUE;
+			}
+			g_object_unref(pixbuf);
+			g_free(filename_short);
 		}
 	}
 
@@ -286,7 +290,7 @@ rs_batch_remove_from_queue(RS_QUEUE *queue, const gchar *filename, gint setting_
 	gboolean ret = FALSE;
 	GtkTreeIter iter;
 
-	gchar *filename_temp = "init";
+	gchar *filename_temp = NULL;
 	gint setting_id_temp;
 
 	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(queue->list), &iter);
@@ -308,6 +312,7 @@ rs_batch_remove_from_queue(RS_QUEUE *queue, const gchar *filename, gint setting_
 					ret = TRUE;
 				}
 			}
+			g_free(filename_temp);
 		} while (gtk_tree_model_iter_next(queue->list, &iter));
 	}
 
@@ -319,6 +324,7 @@ rs_batch_remove_from_queue(RS_QUEUE *queue, const gchar *filename, gint setting_
 static gboolean
 batch_exists_in_queue(RS_QUEUE *queue, const gchar *filename, gint setting_id)
 {
+	gboolean ret = FALSE;
 	GtkTreeIter iter;
 
 	gchar *filename_temp;
@@ -338,13 +344,12 @@ batch_exists_in_queue(RS_QUEUE *queue, const gchar *filename, gint setting_id)
 			if (g_str_equal(filename, filename_temp))
 			{
 				if (setting_id == setting_id_temp)
-					return TRUE;
+					ret = TRUE;
 			}
-		} while (gtk_tree_model_iter_next(queue->list, &iter));
-		return FALSE;
+			g_free(filename_temp);
+		} while (gtk_tree_model_iter_next(queue->list, &iter) && !ret);
 	}
-	else
-		return FALSE;
+	return ret;
 }
 
 static gboolean
@@ -386,6 +391,13 @@ rs_batch_process(RS_QUEUE *queue)
 	gboolean abort_render = FALSE;
 	gboolean fullscreen = FALSE;
 	RS_COLOR_TRANSFORM *rct = rs_color_transform_new();
+	GTimeVal start_time;
+	GTimeVal now_time = {0,0};
+	gint time, eta;
+	GtkWidget *eta_label = gtk_label_new(NULL);
+	gchar *eta_text;
+	gint h = 0, m = 0, s = 0;
+	gint done = 0, left = 0;
 
 	/* Initialize dimensions */
 	switch (queue->size_lock)
@@ -418,6 +430,7 @@ rs_batch_process(RS_QUEUE *queue)
 	gtk_container_add (GTK_CONTAINER (window), vbox);
 	gtk_box_pack_start (GTK_BOX (vbox), gui_framed(preview, _("Last image:"), GTK_SHADOW_IN), TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), eta_label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), cancel, FALSE, FALSE, 0);
 
 	/* Remember fullscreen state */
@@ -431,8 +444,34 @@ rs_batch_process(RS_QUEUE *queue)
 	while (gtk_events_pending()) gtk_main_iteration();
 
 	g_mkdir_with_parents(queue->directory, 00755);
+
+	g_get_current_time(&start_time);
+	left = rs_batch_num_entries(queue);
+
 	while(gtk_tree_model_get_iter_first(queue->list, &iter) && (!abort_render))
 	{
+		done++;
+		if (now_time.tv_sec > 0)
+		{
+			time = (gint) (now_time.tv_sec-start_time.tv_sec);
+			eta = (time/done)*left;
+			h = (eta/3600);
+			eta %= 3600;
+			m = (eta/60);
+			eta %= 60;
+			s = eta;
+
+			eta_text = g_strdup_printf(_("Time left: %dh %dm %ds"), h, m, s);
+		}
+		else
+		{
+			eta_text = g_strdup(_("Time left: ..."));
+		}
+		left--;
+
+		gtk_label_set_text(GTK_LABEL(eta_label), eta_text);
+		g_free(eta_text);
+
 		gtk_tree_model_get(queue->list, &iter,
 			RS_QUEUE_ELEMENT_FILENAME, &filename_in,
 			RS_QUEUE_ELEMENT_SETTING_ID, &setting_id,
@@ -500,12 +539,14 @@ rs_batch_process(RS_QUEUE *queue)
 					width, height, TRUE, scale, setting_id, queue->cms);
 				g_free(parsed_filename);
 				g_string_free(filename, TRUE);
-				rs_photo_free(photo);
+				g_object_unref(photo);
 			}
 			photo = NULL;
 		}
 		gtk_list_store_remove(GTK_LIST_STORE(queue->list), &iter);
 		batch_queue_save(queue);
+
+		g_get_current_time(&now_time);
 	}
 	rs_color_transform_free(rct);
 	gtk_widget_destroy(window);
@@ -623,11 +664,11 @@ make_batchbuttons(RS_QUEUE *queue)
 		g_signal_connect ((gpointer) start_button, "clicked", G_CALLBACK (batch_button_start_clicked), queue);
 
 		remove_button = gtk_button_new();
-		gtk_button_set_label(GTK_BUTTON(remove_button), "Remove");
+		gtk_button_set_label(GTK_BUTTON(remove_button), _("Remove"));
 		g_signal_connect ((gpointer) remove_button, "clicked", G_CALLBACK (batch_button_remove_clicked), queue);
 
 		remove_all_button = gtk_button_new();
-		gtk_button_set_label(GTK_BUTTON(remove_all_button), "Remove all");
+		gtk_button_set_label(GTK_BUTTON(remove_all_button), _("Remove all"));
 		g_signal_connect ((gpointer) remove_all_button, "clicked", G_CALLBACK (batch_button_remove_all_clicked), queue);
 
 		gtk_box_pack_start(GTK_BOX (box), start_button, FALSE, FALSE, 0);
@@ -742,6 +783,7 @@ batch_size_selection(GtkWidget *button, RS_QUEUE *queue)
 		/* Leave the window at its last position */
 		gtk_window_set_position(GTK_WINDOW(queue->size_window), GTK_WIN_POS_NONE);
 		gtk_widget_show(queue->size_window);
+		gtk_window_present(GTK_WINDOW(queue->size_window));
 		return;
 	}
 
@@ -813,6 +855,7 @@ batch_size_selection(GtkWidget *button, RS_QUEUE *queue)
 
 	gtk_container_add (GTK_CONTAINER (queue->size_window), vbox);
 	gtk_widget_show_all(queue->size_window);
+	gtk_window_present(GTK_WINDOW(queue->size_window));
 
 	size_lockbox_changed(gui_confbox_get_active(lockbox), queue);
 	gui_confbox_set_callback(lockbox, queue, size_lockbox_changed);
@@ -826,16 +869,17 @@ size_update_infolabel(RS_QUEUE *queue)
 	switch (queue->size_lock)
 	{
 		case LOCK_WIDTH:
-			g_string_printf(gs, "Constant width:\n%d", queue->width);
+			g_string_printf(gs, _("Constant width:\n%d"), queue->width);
 			break;
 		case LOCK_HEIGHT:
-			g_string_printf(gs, "Constant height:\n%d", queue->height);
+			g_string_printf(gs, _("Constant height:\n%d"), queue->height);
 			break;
 		case LOCK_SCALE:
-			g_string_printf(gs, "Constant Scale:\n%d", queue->scale);
+			g_string_printf(gs, _("Constant Scale:\n%d"), queue->scale);
+			g_string_append(gs, "%"); /* FIXME: merge with the above line after release */
 			break;
 		case LOCK_BOUNDING_BOX:
-			g_string_printf(gs, "Maximum size:\n%d x %d", queue->width, queue->height);
+			g_string_printf(gs, _("Maximum size:\n%d x %d"), queue->width, queue->height);
 			break;
 	}
 
@@ -903,3 +947,26 @@ make_batchbox(RS_QUEUE *queue)
 
 	return batchbox;
 }
+
+/**
+ * Returns the number of entries in the batch queue
+ * @param queue A RS_QUEUE
+ * @return The number of entries in the queue
+ */
+gint
+rs_batch_num_entries(RS_QUEUE *queue)
+{
+	gint num = 0;
+	GtkTreeIter iter;
+
+	gtk_tree_model_get_iter_first(queue->list, &iter);
+
+	if (gtk_list_store_iter_is_valid(GTK_LIST_STORE(queue->list), &iter))
+	{
+		do
+		{
+			num++;
+		} while (gtk_tree_model_iter_next(queue->list, &iter));
+	}
+	return num;
+}
