@@ -22,6 +22,7 @@
 #include <libxml/xmlwriter.h>
 #include "rawstudio.h"
 #include "rs-cache.h"
+#include "rs-photo.h"
 
 /* This will be written to XML files for making backward compatibility easier to implement */
 #define CACHEVERSION 2
@@ -94,6 +95,8 @@ rs_cache_save(RS_PHOTO *photo)
 			photo->settings[id]->warmth);
 		xmlTextWriterWriteFormatElement(writer, BAD_CAST "tint", "%f",
 			photo->settings[id]->tint);
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "sharpen", "%f",
+			photo->settings[id]->sharpen);
 		if (photo->settings[id]->curve_nknots > 0)
 		{
 			xmlTextWriterStartElement(writer, BAD_CAST "curve");
@@ -133,6 +136,8 @@ rs_cache_load_setting(RS_SETTINGS_DOUBLE *rss, xmlDocPtr doc, xmlNodePtr cur)
 			target = &rss->warmth;
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "tint")))
 			target = &rss->tint;
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "sharpen")))
+			target = &rss->sharpen;
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "curve")))
 		{
 			gchar **vals;
@@ -189,6 +194,7 @@ rs_cache_load(RS_PHOTO *photo)
 	gchar *cachename;
 	gint id;
 	gint version = 0;
+	RS_SETTINGS_DOUBLE *settings;
 
 	cachename = rs_cache_get_name(photo->filename);
 	if (!cachename) return(FALSE);
@@ -216,7 +222,10 @@ rs_cache_load(RS_PHOTO *photo)
 			xmlFree(val);
 			if (id>2) id=0;
 			if (id<0) id=0;
-			rs_cache_load_setting(photo->settings[id], doc, cur->xmlChildrenNode);
+			settings = rs_settings_double_new();
+			rs_cache_load_setting(settings, doc, cur->xmlChildrenNode);
+			rs_photo_apply_settings_double(photo, id, settings, MASK_ALL);
+			rs_settings_double_free(settings);
 		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "priority")))
 		{
@@ -245,43 +254,44 @@ rs_cache_load(RS_PHOTO *photo)
 		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "crop")))
 		{
+			RS_RECT *crop = g_new0(RS_RECT, 1);
 			gchar **vals;
 
-			if (!photo->crop)
-				photo->crop = (RS_RECT *) g_malloc(sizeof(RS_RECT));
-			
 			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
 			vals = g_strsplit((gchar *)val, " ", 4);
 			if (vals[0])
 			{
-				photo->crop->x1 = atoi((gchar *) vals[0]);
+				crop->x1 = atoi((gchar *) vals[0]);
 				if (vals[1])
 				{
-					photo->crop->y1 = atoi((gchar *) vals[1]);
+					crop->y1 = atoi((gchar *) vals[1]);
 					if (vals[2])
 					{
-						photo->crop->x2 = atoi((gchar *) vals[2]);
+						crop->x2 = atoi((gchar *) vals[2]);
 						if (vals[3])
-							photo->crop->y2 = atoi((gchar *) vals[3]);
+							crop->y2 = atoi((gchar *) vals[3]);
 					}
 				}
 			}
+
+			/* If crop was done before demosaic was implemented, we should
+			   double the dimensions */
+			if (version < 2)
+			{
+				crop->x1 *= 2;
+				crop->y1 *= 2;
+				crop->x2 *= 2;
+				crop->y2 *= 2;
+			}
+
+			rs_photo_set_crop(photo, crop);
+			g_free(crop);
 			g_strfreev(vals);
 			xmlFree(val);
 		}
 		cur = cur->next;
 	}
 
-	/* If crop was done before demosaic was implemented, we should
-	   double the dimensions */
-	if ((version < 2) && (photo->crop != NULL))
-	{
-		photo->crop->x1 *= 2;
-		photo->crop->y1 *= 2;
-		photo->crop->x2 *= 2;
-		photo->crop->y2 *= 2;
-	}
-
 	xmlFreeDoc(doc);
 	g_free(cachename);
 	return(TRUE);
@@ -295,12 +305,26 @@ rs_cache_load_quick(const gchar *filename, gint *priority, gboolean *exported)
 	xmlChar *val;
 	gchar *cachename;
 
-	if (!filename) return;
+	if (!filename)
+		return;
+
 	cachename = rs_cache_get_name(filename);
-	if (!cachename) return;
-	if (!g_file_test(cachename, G_FILE_TEST_IS_REGULAR)) return;
+
+	if (!cachename)
+		return;
+
+	if (!g_file_test(cachename, G_FILE_TEST_IS_REGULAR))
+	{
+		g_free(cachename);
+		return;
+	}
+
 	doc = xmlParseFile(cachename);
-	if(doc==NULL) return;
+	g_free(cachename);
+
+	if(doc==NULL)
+		return;
+
 	if (exported) *exported = FALSE;
 
 	cur = xmlDocGetRootElement(doc);
@@ -325,7 +349,6 @@ rs_cache_load_quick(const gchar *filename, gint *priority, gboolean *exported)
 	}
 	
 	xmlFreeDoc(doc);
-	g_free(cachename);
 	return;
 }
 
@@ -379,7 +402,7 @@ rs_cache_save_flags(const gchar *filename, const guint *priority, const gboolean
 
 	/* Free the photo */
 	photo->filename = NULL;
-	rs_photo_free(photo);
+	g_object_unref(photo);
 
 	return;
 }
