@@ -48,6 +48,7 @@ struct _RS_COLOR_TRANSFORM_PRIVATE {
 	guint pixelsize;
 	gfloat *curve __deprecated;
 	RS_MATRIX4 color_matrix;
+	RS_MATRIX4 adobe_matrix;
 	gfloat previewtable[65536] __deprecated;
 	guchar gammatable8[65536] __deprecated;
 	gushort gammatable16[65536] __deprecated;
@@ -82,6 +83,7 @@ rs_color_transform_new()
 		rct->priv->pre_mul[i] = 1.0;
 	rct->priv->bits_per_color = 8;
 	matrix4_identity(&rct->priv->color_matrix);
+	matrix4_identity(&rct->priv->adobe_matrix);
 	rct->priv->transform = NULL;
 	rct->priv->spline = NULL;
 	rct->priv->nknots = 0;
@@ -194,16 +196,18 @@ rs_color_transform_set_from_settings(RS_COLOR_TRANSFORM *rct, RS_SETTINGS_DOUBLE
 
 	g_assert(rct != NULL);
 
-	matrix4_identity(&rct->priv->color_matrix);
+	if (mask & (MASK_EXPOSURE|MASK_SATURATION|MASK_HUE))
+	{
+		/* FIXME: this is broken, we should cache the results */
+		if (rct->priv->transform)
+			matrix4_identity(&rct->priv->color_matrix);
+		else
+			rct->priv->color_matrix = rct->priv->adobe_matrix;
 
-	if (mask & MASK_EXPOSURE)
 		matrix4_color_exposure(&rct->priv->color_matrix, settings->exposure);
-
-	if (mask & MASK_SATURATION)
 		matrix4_color_saturate(&rct->priv->color_matrix, settings->saturation);
-
-	if (mask & MASK_HUE)
 		matrix4_color_hue(&rct->priv->color_matrix, settings->hue);
+	}
 
 	if (mask & MASK_WB)
 	{
@@ -299,6 +303,15 @@ rs_color_transform_set_cms_transform(RS_COLOR_TRANSFORM *rct, void *transform)
 	select_render(rct);
 }
 
+void
+rs_color_transform_set_adobe_matrix(RS_COLOR_TRANSFORM *rct, RS_MATRIX4 *matrix)
+{
+	g_assert(rct != NULL);
+	g_assert(matrix != NULL);
+
+	rct->priv->adobe_matrix = *matrix;
+}
+
 static void
 make_tables(RS_COLOR_TRANSFORM *rct)
 {
@@ -431,6 +444,9 @@ COLOR_TRANSFORM(transform_nocms_float)
 	register gint r,g,b;
 	gfloat r1, r2, g1, g2, b1, b2;
 
+	if ((rct==NULL) || (width<1) || (height<1) || (in == NULL) || (in_rowstride<8) || (out == NULL) || (out_rowstride<1))
+		return;
+
 	for(y=0 ; y<height ; y++)
 	{
 		guchar *d8 = out + y * out_rowstride;
@@ -516,6 +532,10 @@ COLOR_TRANSFORM(transform_nocms8_sse)
 			+ rct->priv->color_matrix.coeff[2][1]
 			+ rct->priv->color_matrix.coeff[2][2])
 	};
+
+	if ((rct==NULL) || (width<1) || (height<1) || (in == NULL) || (in_rowstride<8) || (out == NULL) || (out_rowstride<1))
+		return;
+
 	asm volatile (
 		"movups (%2), %%xmm2\n\t" /* rs->pre_mul */
 		"movaps (%0), %%xmm3\n\t" /* matrix */
@@ -610,6 +630,10 @@ COLOR_TRANSFORM(transform_nocms8_3dnow)
 	mat[11] = 0.0;
 	top[0] = 65535.0;
 	top[1] = 65535.0;
+
+	if ((rct==NULL) || (width<1) || (height<1) || (in == NULL) || (in_rowstride<8) || (out == NULL) || (out_rowstride<1))
+		return;
+
 	asm volatile (
 		"femms\n\t"
 		"pxor %%mm7, %%mm7\n\t" /* 0x0 */
@@ -712,6 +736,10 @@ COLOR_TRANSFORM(transform_cms8_sse)
 		rct->priv->color_matrix.coeff[1][2],
 		rct->priv->color_matrix.coeff[2][2],
 		0.0 };
+
+	if ((rct==NULL) || (width<1) || (height<1) || (in == NULL) || (in_rowstride<8) || (out == NULL) || (out_rowstride<1))
+		return;
+
 	asm volatile (
 		"movups (%2), %%xmm2\n\t" /* rs->pre_mul */
 		"movaps (%0), %%xmm3\n\t" /* matrix */
@@ -806,6 +834,10 @@ COLOR_TRANSFORM(transform_cms8_3dnow)
 	mat[11] = 0.0;
 	top[0] = 65535.0;
 	top[1] = 65535.0;
+
+	if ((rct==NULL) || (width<1) || (height<1) || (in == NULL) || (in_rowstride<8) || (out == NULL) || (out_rowstride<1))
+		return;
+
 	asm volatile (
 		"femms\n\t"
 		"pxor %%mm7, %%mm7\n\t" /* 0x0 */
@@ -899,6 +931,9 @@ COLOR_TRANSFORM(transform_cms_c)
 	gint pre_muli[4];
 	RS_MATRIX4Int mati;
 
+	if ((rct==NULL) || (width<1) || (height<1) || (in == NULL) || (in_rowstride<8) || (out == NULL) || (out_rowstride<1))
+		return;
+
 	matrix4_to_matrix4int(&rct->priv->color_matrix, &mati);
 	for(x=0;x<4;x++)
 		pre_muli[x] = (gint) (rct->priv->pre_mul[x]*128.0);
@@ -942,6 +977,9 @@ COLOR_TRANSFORM(transform_nocms_c)
 	gint pre_muli[4];
 	RS_MATRIX4Int mati;
 
+	if ((rct==NULL) || (width<1) || (height<1) || (in == NULL) || (in_rowstride<8) || (out == NULL) || (out_rowstride<1))
+		return;
+
 	matrix4_to_matrix4int(&rct->priv->color_matrix, &mati);
 	for(x=0;x<4;x++)
 		pre_muli[x] = (gint) (rct->priv->pre_mul[x]*128.0);
