@@ -21,9 +21,12 @@
 #include <math.h>
 #include <libxml/encoding.h>
 #include <libxml/xmlwriter.h>
-
+#include <string.h> /* memset() */
+#include "color.h"
 #include "rs-spline.h"
 #include "rs-curve.h"
+#include "rs-color-transform.h"
+#include "gtk-interface.h"
 
 struct _RSCurveWidget
 {
@@ -33,6 +36,14 @@ struct _RSCurveWidget
 	gfloat *array;
 	guint array_length;
 	gfloat marker;
+	gulong size_signal;
+	guint size_timeout_helper;
+
+	/* For drawing the histogram */
+	guint histogram_data[4][256];
+	guchar *bg_buffer;
+	RS_COLOR_TRANSFORM *rct;
+	RS_SETTINGS_DOUBLE *settings;
 };
 
 struct _RSCurveWidgetClass
@@ -40,17 +51,18 @@ struct _RSCurveWidgetClass
 	GtkDrawingAreaClass parent_class;
 };
 
-static GdkCursor *crosshair = NULL;
-
 static void rs_curve_widget_class_init(RSCurveWidgetClass *klass);
 static void rs_curve_widget_init(RSCurveWidget *curve);
 static void rs_curve_widget_destroy(GtkObject *object);
+static gboolean rs_curve_size_allocate_helper(RSCurveWidget *curve);
+static void rs_curve_size_allocate(GtkWidget *widget, GtkAllocation *allocation, gpointer user_data);
 static void rs_curve_changed(RSCurveWidget *curve);
 static void rs_curve_draw(RSCurveWidget *curve);
 static gboolean rs_curve_widget_expose(GtkWidget *widget, GdkEventExpose *event);
 static gboolean rs_curve_widget_button_press(GtkWidget *widget, GdkEventButton *event);
 static gboolean rs_curve_widget_button_release(GtkWidget *widget, GdkEventButton *event);
 static gboolean rs_curve_widget_motion_notify(GtkWidget *widget, GdkEventMotion *event);
+static void rs_curve_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation, gpointer user_data);
 
 enum {
   CHANGED_SIGNAL,
@@ -108,14 +120,20 @@ rs_curve_widget_init(RSCurveWidget *curve)
 	curve->array_length = 0;
 	curve->spline = rs_spline_new(NULL, 0, NATURAL);
 	curve->marker = -1.0;
+	curve->bg_buffer = NULL;
+	curve->settings = rs_settings_double_new();
+	curve->settings->saturation = 0.0f; /* We want the histogram to be desaturated */
+	curve->rct = rs_color_transform_new();
+	rs_color_transform_set_gamma(curve->rct, GAMMA);
 
 	/* Let us know about pointer movements */
 	gtk_widget_set_events(GTK_WIDGET(curve), 0
 		| GDK_BUTTON_PRESS_MASK
 		| GDK_BUTTON_RELEASE_MASK
 		| GDK_POINTER_MOTION_MASK);
-	if (!crosshair)
-		crosshair = gdk_cursor_new(GDK_CROSSHAIR);
+
+	curve->size_signal = g_signal_connect(curve, "size-allocate", G_CALLBACK(rs_curve_size_allocate), NULL);
+	g_signal_connect(G_OBJECT(curve), "size-allocate", G_CALLBACK(rs_curve_widget_size_allocate), NULL);
 }
 
 /**
@@ -172,6 +190,26 @@ rs_curve_widget_set_array(RSCurveWidget *curve, gfloat *array, guint array_lengt
 	}
 }
 
+/**
+ * Draw a histogram in the background of the widget
+ * @param curve A RSCurveWidget
+ * @param image A image to sample from
+ * @param setting Settings to use, curve and saturation will be ignored
+ */
+void
+rs_curve_draw_histogram(RSCurveWidget *curve, RS_IMAGE16 *image, RS_SETTINGS_DOUBLE *settings)
+{
+	rs_settings_double_copy(settings, curve->settings, MASK_ALL-MASK_CURVE);
+	rs_color_transform_set_from_settings(curve->rct, curve->settings, MASK_ALL);
+	rs_color_transform_make_histogram(curve->rct, image, curve->histogram_data);
+
+	if (curve->bg_buffer)
+		g_free(curve->bg_buffer);
+	curve->bg_buffer = NULL;
+
+	rs_curve_draw(curve);
+}
+
 /**
  * Instance destruction
  */
@@ -257,37 +295,11 @@ rs_curve_widget_move_knot(RSCurveWidget *curve, gint knot, gfloat x, gfloat y)
 gfloat *
 rs_curve_widget_sample(RSCurveWidget *curve, gfloat *samples, guint nbsamples)
 {
-	gfloat *knots = NULL;
-	guint nknots;
-	gint i;
-
 	g_return_val_if_fail (curve != NULL, NULL);
 	g_return_val_if_fail (RS_IS_CURVE_WIDGET(curve), NULL);
 
-	/* Get the knots from the spline */
-	rs_spline_get_knots(curve->spline, &knots, &nknots);
+	samples = rs_spline_sample(curve->spline, samples, nbsamples);
 
-	if ((nknots>1) && knots)
-	{
-		/* Find the sample number for first and last knot */
-		const gint start = knots[0*2+0]*((gfloat)nbsamples);
-		const gint stop = knots[(nknots-1)*2+0]*((gfloat)nbsamples);
-
-		/* Allocate space for output, if not given */
-		if (!samples)
-			samples = g_new(gfloat, nbsamples);
-
-		/* Sample between knots */
-		rs_spline_sample(curve->spline, samples+start, stop-start);
-
-		/* Sample flat curve before first knot */
-		for(i=0;i<start;i++)
-			samples[i] = knots[0*2+1];
-
-		/* Sample flat curve after last knot */
-		for(i=stop;i<nbsamples;i++)
-			samples[i] = knots[(nknots-1)*2+1];
-	}
 	return(samples);
 }
 
@@ -398,8 +410,8 @@ rs_curve_widget_load(RSCurveWidget *curve, const gchar *filename)
 					vals = g_strsplit((gchar *)val, " ", 4);
 					if (vals[0] && vals[1])
 					{
-						x = atof((gchar *) vals[0]);
-						y = atof((gchar *) vals[1]);
+						x = g_strtod(vals[0], NULL);
+						y = g_strtod(vals[1], NULL);
 						rs_curve_widget_add_knot(curve, x,y);
 					}
 					g_strfreev(vals);
@@ -416,7 +428,7 @@ rs_curve_widget_load(RSCurveWidget *curve, const gchar *filename)
 }
 
 /* Background color */
-static const GdkColor darkgrey = {0, 0xaaaa, 0xaaaa, 0xaaaa};
+static const GdkColor darkgrey = {0, 0x7777, 0x7777, 0x7777};
 
 /* Foreground color */
 static const GdkColor lightgrey = {0, 0xcccc, 0xcccc, 0xcccc};
@@ -463,36 +475,93 @@ rs_curve_draw_marker(GtkWidget *widget)
 }
 
 static void
-rs_curve_draw_background(GdkDrawable *window)
+rs_curve_draw_background(GtkWidget *widget)
 {
-	/* Iterator */
-	gint i;
+	gint i, max = 0, x, y;
 
 	/* Width */
 	gint width;
 
 	/* Height */
 	gint height;
+	RSCurveWidget *curve;
+	GdkDrawable *window;
+	GdkGC *gc;
+
+	/* Get back our curve widget */
+	curve = RS_CURVE_WIDGET(widget);
+
+	/* Get the drawing window */
+	window = GDK_DRAWABLE(widget->window);
 
 	if (!window) return;
 
 	/* Graphics context */
-	GdkGC *gc = gdk_gc_new(window);
+	gc = gdk_gc_new(window);
 
 	/* Width and height */
 	gdk_drawable_get_size(window, &width, &height);
 
-	/* Prepare the graphics context */
-	gdk_gc_set_rgb_fg_color(gc, &lightgrey);
+	/* Scaled histogram */
+	gint hist[width];
 
-	/* Clear the window */
-	gdk_draw_rectangle(window, gc, TRUE,
-			   0, 0,
-			   width, height);
+	if (!curve->bg_buffer)
+	{
+		curve->bg_buffer = g_new(guchar, width*height*4);
+
+		/* Clear the window */
+		memset(curve->bg_buffer, 0x99, width*height*4);
+
+		/* Prepare histogram */
+		if (curve->histogram_data)
+		{
+			/* find the max value */
+			/* Except 0 and 255! */
+			for (i = 1; i < 255; i++)
+				if (curve->histogram_data[3][i] > max)
+					max = curve->histogram_data[3][i];
+
+			/* Find height scale factor */
+			gfloat factor = (gfloat)(max+height)/(gfloat)height;
+
+			/* Find width scale factor */
+			gfloat source, scale = 253.0/width;
+			gint source1, source2;
+			gfloat weight1, weight2;
+			for (i = 0; i < width; i++)
+			{
+				source = ((gdouble)i)*scale;
+				source1 = floor(source);
+				source2 = source1+1;
+				weight1 = 1.0 - (source-source1);
+				weight2 = 1.0 - weight1;
+
+				hist[i] = (curve->histogram_data[3][1+source1] * weight1
+					+ curve->histogram_data[3][1+source2] * weight2)/factor;
+			}
+
+			for (x = 0; x < width; x++)
+			{
+				for (y = 0; y < hist[x]; y++)
+				{
+					guchar *p = curve->bg_buffer + ((height-1)-y) * width*3 + x * 3;
+					p[R] = 0xB0;
+					p[G] = 0xB0;
+					p[B] = 0xB0;
+				}
+			}
+		}
+	}
+
+	/* Prepare the graphics context */
+	gdk_gc_set_rgb_fg_color(gc, &darkgrey);
+
+	/* Draw histogram to screen */
+	gdk_draw_rgb_image(window, gc, 0, 0, width, height, GDK_RGB_DITHER_NONE, curve->bg_buffer, width*3);
 
 	/* Draw all lines */
-	gdk_gc_set_rgb_fg_color(gc, &darkgrey);
-	for (i=0; i<=4; i++) {
+	for (i=0; i<=4; i++)
+	{
 		gint x = i*width/4;
 		gint y = i*height/4;
 		gdk_draw_line(window, gc, x, 0, x, height);
@@ -547,7 +616,7 @@ rs_curve_draw_knots(GtkWidget *widget)
 		gint x = (gint)(knots[2*curve->active_knot + 0]*width);
 		gint y = (gint)(height*(1-knots[2*curve->active_knot + 1]));
 		gdk_gc_set_rgb_fg_color(gc, &red);
-		gdk_draw_rectangle(window, gc, TRUE, x-2, y-2, 4, 4);
+		gdk_draw_rectangle(window, gc, FALSE, x-3, y-3, 6, 6);
 	}
 
 	g_free(knots);
@@ -589,7 +658,11 @@ rs_curve_draw_spline(GtkWidget *widget)
 
 	for (i=0; i<width; i++)
 	{
-		gint y = (gint)(height*(1-samples[i]));
+		gint y = (gint)(height*(1-samples[i])+0.5);
+		if (y < 0)
+			y = 0;
+		else if (y > (height-1))
+			y = height-1;
 		gdk_draw_point(window, gc, i, y);
 	}
 
@@ -610,7 +683,7 @@ rs_curve_draw(RSCurveWidget *curve)
 	if (GTK_WIDGET_VISIBLE(widget))
 	{
 		/* Draw the background */
-		rs_curve_draw_background(widget->window);
+		rs_curve_draw_background(widget);
 
 		/* Draw the marker line */
 		rs_curve_draw_marker(widget);
@@ -622,6 +695,45 @@ rs_curve_draw(RSCurveWidget *curve)
 		rs_curve_draw_spline(widget);
 	}
 }
+  
+static gboolean
+rs_curve_size_allocate_helper(RSCurveWidget *curve)
+{
+	gboolean ret = FALSE;
+
+	gdk_threads_enter();
+	if (gtk_events_pending())
+		ret = TRUE;
+	else
+	{
+		if (GTK_WIDGET(curve)->allocation.width != GTK_WIDGET(curve)->allocation.height)
+		{
+			g_signal_handler_block(RS_CURVE_WIDGET(curve), RS_CURVE_WIDGET(curve)->size_signal);
+			gtk_widget_set_size_request(GTK_WIDGET(curve), -1, GTK_WIDGET(curve)->allocation.width);
+			GUI_CATCHUP();
+			g_signal_handler_unblock(RS_CURVE_WIDGET(curve), RS_CURVE_WIDGET(curve)->size_signal);
+		}
+		curve->size_timeout_helper = 0;
+	}
+	gdk_threads_leave();
+
+	return ret;
+}
+
+/**
+ * Make the curve widget squared
+ */
+static void
+rs_curve_size_allocate(GtkWidget *widget, GtkAllocation *allocation, gpointer user_data)
+{
+	RSCurveWidget *curve = RS_CURVE_WIDGET(widget);
+
+	if (allocation->width != allocation->height)
+	{
+		if (curve->size_timeout_helper == 0)
+			curve->size_timeout_helper = g_timeout_add(50, (GSourceFunc) rs_curve_size_allocate_helper, curve);
+	}
+}
 
 /**
  * Propagate changes
@@ -653,9 +765,6 @@ rs_curve_widget_expose(GtkWidget *widget, GdkEventExpose *event)
 	if (event->count > 0)
 		return FALSE;
 
-	/* Set crosshair cursor */
-	gdk_window_set_cursor(widget->window, crosshair);
-
 	rs_curve_draw(RS_CURVE_WIDGET(widget));
 
 	return FALSE;
@@ -799,12 +908,26 @@ rs_curve_widget_motion_notify(GtkWidget *widget, GdkEventMotion *event)
 
 	/* Update knots if needed */
 	if (old_active_knot != curve->active_knot)
-		rs_curve_draw_knots(widget);
+		rs_curve_draw(RS_CURVE_WIDGET(widget));
 	g_free(knots);
 
 	return(TRUE);
 }
 
+static void
+rs_curve_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation, gpointer user_data)
+{
+	/* Get back our curve widget */
+	RSCurveWidget *curve = RS_CURVE_WIDGET(widget);
+
+	/* Free our bg_buffer, since it must be useless by now */
+	if (curve->bg_buffer)
+		g_free(curve->bg_buffer);
+
+	/* Mark it as not existing */
+	curve->bg_buffer = NULL;
+}
+
 #ifdef RS_CURVE_TEST
 
 void
