@@ -30,17 +30,16 @@
 #include "rs-image.h"
 
 struct struct_program {
-	gint offset;
-	gint scale;
+	gint divisor;
+	gint scale[9];
 };
 
 static void rs_image16_rotate(RS_IMAGE16 *rsi, gint quarterturns);
 static void rs_image16_mirror(RS_IMAGE16 *rsi);
 static void rs_image16_flip(RS_IMAGE16 *rsi);
-inline static void rs_image16_nearest(RS_IMAGE16 *in, gushort *out, gdouble x, gdouble y);
-inline static void rs_image16_bilinear(RS_IMAGE16 *in, gushort *out, gdouble x, gdouble y);
+inline static void rs_image16_nearest(RS_IMAGE16 *in, gushort *out, gint x, gint y);
+inline static void rs_image16_bilinear(RS_IMAGE16 *in, gushort *out, gint x, gint y);
 inline static void rs_image16_bicubic(RS_IMAGE16 *in, gushort *out, gdouble x, gdouble y);
-static void rs_image8_realloc(RS_IMAGE8 *rsi, const guint width, const guint height, const guint channels, const guint pixelsize);
 inline gushort topright(gushort *in, struct struct_program *program, gint divisor);
 inline gushort top(gushort *in, struct struct_program *program, gint divisor);
 inline gushort topleft(gushort *in, struct struct_program *program, gint divisor);
@@ -117,31 +116,6 @@ rs_image16_init (RS_IMAGE16 *self)
 	self->pixels = NULL;
 }
 
-void
-rs_image8_ref(RS_IMAGE8 *image)
-{
-	if (!image) return;
-
-	g_static_mutex_lock(&giant_spinlock);
-	image->reference_count++;
-	g_static_mutex_unlock(&giant_spinlock);
-}
-
-void
-rs_image8_unref(RS_IMAGE8 *image)
-{
-	if (!image) return;
-
-	g_static_mutex_lock(&giant_spinlock);
-	image->reference_count--;
-	if (image->reference_count < 1)
-	{
-		g_free(image->pixels);
-		g_free(image);
-	}
-	g_static_mutex_unlock(&giant_spinlock);
-}
-
 void
 rs_image16_orientation(RS_IMAGE16 *rsi, const gint orientation)
 {
@@ -230,6 +204,7 @@ rs_image16_rotate(RS_IMAGE16 *rsi, gint quarterturns)
 		default:
 			break;
 	}
+	g_signal_emit(rsi, signals[PIXELDATA_CHANGED], 0, NULL);
 	rs_image16_unref(rsi);
 	return;
 }
@@ -259,6 +234,7 @@ rs_image16_mirror(RS_IMAGE16 *rsi)
 	}
 	ORIENTATION_MIRROR(rsi->orientation);
 
+	g_signal_emit(rsi, signals[PIXELDATA_CHANGED], 0, NULL);
 	rs_image16_unref(rsi);
 }
 
@@ -282,22 +258,24 @@ rs_image16_flip(RS_IMAGE16 *rsi)
 	g_free(tmp);
 	ORIENTATION_FLIP(rsi->orientation);
 
+	g_signal_emit(rsi, signals[PIXELDATA_CHANGED], 0, NULL);
 	rs_image16_unref(rsi);
 
 	return;
 }
 
 static void inline
-rs_image16_preview(RS_IMAGE16 *in, gushort *out, gdouble x, gdouble y)
+rs_image16_preview(RS_IMAGE16 *in, gushort *out, gint x, gint y)
 {
-	const gint nx = (lrint(x)&0xfffffff8)+4;
-	const gint ny = (lrint(y)&0xfffffff8)+4;
-	const gushort *a = GET_PIXEL(in, nx, ny);
+	const gint nx = ((x>>11)<<3)+4;
+	const gint ny = ((y>>11)<<3)+4;
 
-	if (unlikely((nx>(in->w-1))||(ny>(in->h-1))))
+	if (unlikely((nx>=(in->w))||(ny>=(in->h))))
 		return;
 	else if (unlikely(nx<0) || unlikely(ny<0))
 		return;
+	
+	const gushort *a = GET_PIXEL(in, nx, ny);
 
 	out[R] = a[R];
 	out[G] = a[G];
@@ -305,10 +283,10 @@ rs_image16_preview(RS_IMAGE16 *in, gushort *out, gdouble x, gdouble y)
 }
 
 static void inline
-rs_image16_nearest(RS_IMAGE16 *in, gushort *out, gdouble x, gdouble y)
+rs_image16_nearest(RS_IMAGE16 *in, gushort *out, gint x, gint y)
 {
-	const gint nx = lrint(x);
-	const gint ny = lrint(y);
+	const gint nx = x>>8;
+	const gint ny = y>>8;
 	const gushort *a = &in->pixels[ny*in->rowstride + nx*in->pixelsize];
 	out[R] = a[R];
 	out[G] = a[G];
@@ -317,15 +295,16 @@ rs_image16_nearest(RS_IMAGE16 *in, gushort *out, gdouble x, gdouble y)
 }
 
 static void inline
-rs_image16_bilinear(RS_IMAGE16 *in, gushort *out, gdouble x, gdouble y)
+rs_image16_bilinear(RS_IMAGE16 *in, gushort *out, gint x, gint y)
 {
-	gint fx = floor(x);
-	gint fy = floor(y);
-	const gint nextx = (fx<(in->w-1)) * in->pixelsize;
-	const gint nexty = (fy<(in->h-1));
+	gint fx = x>>8;
+	gint fy = y>>8;
+	gint nextx;
+	gint nexty;
+	
 	gushort *a, *b, *c, *d;
-	gdouble diffx, diffy;
-	gdouble aw, bw, cw, dw;
+	gint diffx, diffy, inv_diffx, inv_diffy;
+	gint aw, bw, cw, dw;
 
 	if (unlikely((fx>(in->w-1))||(fy>(in->h-1))))
 		return;
@@ -336,7 +315,7 @@ rs_image16_bilinear(RS_IMAGE16 *in, gushort *out, gdouble x, gdouble y)
 		else
 		{
 			fx = 0;
-			x = 0.0;
+			x = 0;
 		}
 	}
 
@@ -347,27 +326,43 @@ rs_image16_bilinear(RS_IMAGE16 *in, gushort *out, gdouble x, gdouble y)
 		else
 		{
 			fy = 0;
-			y = 0.0;
+			y = 0;
 		}
 	}
-
+	
+	if (fx < in->w-1)
+		nextx = in->pixelsize;
+	else
+		nextx = 0;
+	
+	if (fy < in->h-1)
+		nexty = in->rowstride;
+	else
+		nexty = 0;
+	
+	fx *= in->pixelsize;
+	fy *= in->rowstride;
+	
 	/* find four cornerpixels */
-	a = &in->pixels[fy*in->rowstride + fx*in->pixelsize];
+	a = &in->pixels[fy + fx];
 	b = a + nextx;
-	c = &in->pixels[(fy+nexty)*in->rowstride + fx*in->pixelsize];
+	c = &in->pixels[fy+nexty + fx];
 	d = c + nextx;
 
 	/* calculate weightings */
-	diffx = x - ((gdouble) fx); /* x distance from a */
-	diffy = y - ((gdouble) fy); /* y distance from a */
-	aw = (1.0-diffx) * (1.0-diffy);
-	bw = diffx       * (1.0-diffy);
-	cw = (1.0-diffx) * diffy;
-	dw = diffx       * diffy;
+	diffx = x&0xff; /* x distance from a */
+	diffy = y&0xff; /* y distance fromy a */
+	inv_diffx = 256 - diffx; /* inverse x distance from a */
+	inv_diffy = 256 - diffy; /* inverse y distance from a */
+	
+	aw = (inv_diffx * inv_diffy) >> 1;  /* Weight is now 0.15 fp */
+	bw = (diffx * inv_diffy) >> 1;
+	cw = (inv_diffx * diffy) >> 1;
+	dw = (diffx * diffy) >> 1;
 
-	out[R]  = (gushort) (a[R]*aw  + b[R]*bw  + c[R]*cw  + d[R]*dw);
-	out[G]  = (gushort) (a[G]*aw  + b[G]*bw  + c[G]*cw  + d[G]*dw);
-	out[B]  = (gushort) (a[B]*aw  + b[B]*bw  + c[B]*cw  + d[B]*dw);
+	out[R]  = (gushort) ((a[R]*aw  + b[R]*bw  + c[R]*cw  + d[R]*dw + 16384) >> 15 );
+	out[G]  = (gushort) ((a[G]*aw  + b[G]*bw  + c[G]*cw  + d[G]*dw + 16384) >> 15 );
+	out[B]  = (gushort) ((a[B]*aw  + b[B]*bw  + c[B]*cw  + d[B]*dw + 16384) >> 15 );
 	out[G2] = 0;
 	return;
 }
@@ -524,6 +519,7 @@ rs_image16_transform(RS_IMAGE16 *in, RS_IMAGE16 *out, RS_MATRIX3 *affine, RS_MAT
 	gdouble w, h;
 	gint row, col;
 	gint destoffset;
+	gint fixed_x, fixed_y;
 
 	rs_image16_ref(in);
 
@@ -616,7 +612,7 @@ rs_image16_transform(RS_IMAGE16 *in, RS_IMAGE16 *out, RS_MATRIX3 *affine, RS_MAT
 	rs_image16_ref(out);
 	/* we use the inverse matrix for this */
 	matrix3_affine_invert(&mat);
-
+	
 	for(row=0;row<out->h;row++)
 	{
 		gdouble foox = ((gdouble)row) * mat.coeff[1][0] + mat.coeff[2][0];
@@ -626,14 +622,16 @@ rs_image16_transform(RS_IMAGE16 *in, RS_IMAGE16 *out, RS_MATRIX3 *affine, RS_MAT
 		{
 			x = ((gdouble)col)*mat.coeff[0][0] + foox;
 			y = ((gdouble)col)*mat.coeff[0][1] + fooy;
-
+			fixed_x = (int)(256.0f * x);
+			fixed_y = (int)(256.0f * y);
 			if (in->preview)
-				rs_image16_preview(in, &out->pixels[destoffset], x, y);
+				rs_image16_preview(in, &out->pixels[destoffset], fixed_x, fixed_y);
 			else
-				rs_image16_bilinear(in, &out->pixels[destoffset], x, y);
+				rs_image16_bilinear(in, &out->pixels[destoffset], fixed_x, fixed_y);
 		}
 	}
 
+	g_signal_emit(out, signals[PIXELDATA_CHANGED], 0, NULL);
 	rs_image16_unref(out);
 	rs_image16_unref(in);
 
@@ -651,7 +649,7 @@ rs_image16_scale_double(RS_IMAGE16 *in, RS_IMAGE16 *out, gdouble scale)
 
 	if ( scale == 1.0 ){
 		rs_image16_ref(in);
-		out = rs_image16_copy(in);
+		out = rs_image16_copy(in, TRUE);
 		rs_image16_unref(in);
 		return(out);
 	}
@@ -781,6 +779,7 @@ rs_image16_scale_double(RS_IMAGE16 *in, RS_IMAGE16 *out, gdouble scale)
 		}
 
 	}
+	g_signal_emit(out, signals[PIXELDATA_CHANGED], 0, NULL);
 	rs_image16_unref(in);
 	rs_image16_unref(out);
 	return(out);
@@ -790,6 +789,16 @@ RS_IMAGE16 *
 rs_image16_new(const guint width, const guint height, const guint channels, const guint pixelsize)
 {
 	RS_IMAGE16 *rsi;
+
+	g_assert(width < 65536);
+	g_assert(height < 65536);
+
+	g_assert(width > 0);
+	g_assert(height > 0);
+
+	g_assert(channels > 0);
+	g_assert(pixelsize >= channels);
+
 	rsi = g_object_new(RS_TYPE_IMAGE16, NULL);
 	rsi->w = width;
 	rsi->h = height;
@@ -801,67 +810,28 @@ rs_image16_new(const guint width, const guint height, const guint channels, cons
 	return(rsi);
 }
 
-RS_IMAGE8 *
-rs_image8_new(const guint width, const guint height, const guint channels, const guint pixelsize)
-{
-	RS_IMAGE8 *rsi;
-
-	rsi = (RS_IMAGE8 *) g_malloc(sizeof(RS_IMAGE8));
-	rsi->w = width;
-	rsi->h = height;
-	ORIENTATION_RESET(rsi->orientation);
-	rsi->rowstride = PITCH(width) * pixelsize;
-	rsi->pixels = g_new0(guchar, rsi->h*rsi->rowstride);
-	rsi->channels = channels;
-	rsi->pixelsize = pixelsize;
-	rsi->reference_count = 0;
-	rs_image8_ref(rsi);
-
-	return(rsi);
-}
-
-static void
-rs_image8_realloc(RS_IMAGE8 *rsi, const guint width, const guint height, const guint channels, const guint pixelsize)
-{
-	if (!rsi) return;
-
-	rs_image8_ref(rsi);
-
-	/* Do we actually differ? */
-	if ((rsi->w != width) || (rsi->h != height) || (rsi->channels != channels) || (rsi->pixelsize != pixelsize))
-	{
-		/* Free the old pixels */
-		g_free(rsi->pixels);
-
-		/* Fill in new values */
-		rsi->w = width;
-		rsi->h = height;
-		rsi->rowstride = PITCH(width) * pixelsize;
-		rsi->pixels = g_new0(guchar, rsi->h*rsi->rowstride);
-		rsi->channels = channels;
-		rsi->pixelsize = pixelsize;
-	}
-
-	rs_image8_unref(rsi);
-}
-
 /**
- * Renders an exposure map on top of an RS_IMAGE8 with 3 channels
- * @param image A RS_IMAGE8
+ * Renders an exposure map on top of an GdkPixbuf with 3 channels
+ * @param pixbuf A GdkPixbuf
  * @param only_row A single row to render or -1 to render all
  */
 void
-rs_image8_render_exposure_mask(RS_IMAGE8 *image, gint only_row)
+gdk_pixbuf_render_exposure_mask(GdkPixbuf *pixbuf, gint only_row)
 {
 	gint row, col;
 	gint start;
 	gint stop;
+	gint height;
+	gint width;
 
-	g_assert(image != NULL);
-	g_assert(image->channels == 3);
+	g_assert(GDK_IS_PIXBUF(pixbuf));
+	g_assert(gdk_pixbuf_get_n_channels(pixbuf) == 3);
 
-	rs_image8_ref(image);
-	if ((only_row > -1) && (only_row < image->h))
+	g_object_ref(pixbuf);
+	width = gdk_pixbuf_get_width(pixbuf);
+	height = gdk_pixbuf_get_height(pixbuf);
+
+	if ((only_row > -1) && (only_row < height))
 	{
 		start = only_row;
 		stop = only_row + 1;
@@ -869,15 +839,15 @@ rs_image8_render_exposure_mask(RS_IMAGE8 *image, gint only_row)
 	else
 	{
 		start = 0;
-		stop = image->h;
+		stop = height;
 	}
 
 	for(row=start;row<stop;row++)
 	{
 		/* Get start pixel of row */
-		guchar *pixel = GET_PIXEL(image, 0, row);
+		guchar *pixel = GET_PIXBUF_PIXEL(pixbuf, 0, row);
 
-		for(col=0;col<image->w;col++)
+		for(col=0;col<width;col++)
 		{
 			/* Catch pixels overexposed and color them red */
 			if ((pixel[R]==0xFF) || (pixel[G]==0xFF) || (pixel[B]==0xFF))
@@ -897,361 +867,154 @@ rs_image8_render_exposure_mask(RS_IMAGE8 *image, gint only_row)
 				pixel += 3;
 		}
 	}
-	rs_image8_unref(image);
-}
-
-/**
- * Renders a shaded image
- * @param in The input image
- * @param out The output image or NULL
- * @return The shaded image
- */
-RS_IMAGE8 *
-rs_image8_render_shaded(RS_IMAGE8 *in, RS_IMAGE8 *out)
-{
-	gint size;
-
-	if (!in) return NULL;
-
-	rs_image8_ref(in);
-
-	if (!out)
-		out = rs_image8_new(in->w, in->h, in->channels, in->pixelsize);
-	else
-		rs_image8_realloc(out, in->w, in->h, in->channels, in->pixelsize);
-
-	rs_image8_ref(out);
-
-	size = in->h * in->rowstride;
-	while(size--)
-		out->pixels[size] = ((in->pixels[size]+63)*3)>>3; /* Magic shade formula :) */
-
-	rs_image8_unref(out);
-	rs_image8_unref(in);
-	return out;
+	g_object_unref(pixbuf);
 }
 
 RS_IMAGE16 *
-rs_image16_copy(RS_IMAGE16 *in)
+rs_image16_copy(RS_IMAGE16 *in, gboolean copy_pixels)
 {
 	RS_IMAGE16 *out;
 	rs_image16_ref(in);
 	out = rs_image16_new(in->w, in->h, in->channels, in->pixelsize);
-	memcpy(out->pixels, in->pixels, in->rowstride*in->h*2);
+	if (copy_pixels)
+		memcpy(out->pixels, in->pixels, in->rowstride*in->h*2);
 	rs_image16_unref(in);
 	return(out);
 }
 
-inline gushort
-topleft(gushort *in, struct struct_program *program, gint divisor)
+void
+convolve_line(RS_IMAGE16 *input, RS_IMAGE16 *output, guint line, struct struct_program *program)
 {
-	gint temp;
-	temp = ((*(in+program[4].offset)) * program[0].scale)
-		+ ((*(in+program[4].offset)) * program[1].scale)
-		+ ((*(in+program[5].offset)) * program[2].scale)
-		+ ((*(in+program[4].offset)) * program[3].scale)
-		+ ((*(in+program[4].offset)) * program[4].scale)
-		+ ((*(in+program[5].offset)) * program[5].scale)
-		+ ((*(in+program[7].offset)) * program[6].scale)
-		+ ((*(in+program[7].offset)) * program[7].scale)
-		+ ((*(in+program[8].offset)) * program[8].scale);
-	temp /= divisor;
-	_CLAMP65535(temp);
-	return(temp);
+	gint size;
+	gint col;
+	gint accu;
+	gushort *line0, *line1, *line2, *dest;
+
+	g_assert(line >= 0);
+	g_assert(line < input->h);
+
+	line0 = GET_PIXEL(input, 0, line-1);
+	line1 = GET_PIXEL(input, 0, line);
+	line2 = GET_PIXEL(input, 0, line+1);
+	dest = GET_PIXEL(output, 0, line);
+
+	/* special case for first line */
+	if (line == 0)
+		line0 = line1;
+
+	/* special case for last line */
+	else if (line == (input->h-1))
+		line2 = line1;
+
+	/* special case for first pixel */
+	for (col = 0; col < input->pixelsize; col++)
+	{
+		accu
+			= program->scale[0] * *line0
+			+ program->scale[1] * *line0
+			+ program->scale[2] * *(line0+input->pixelsize)
+			+ program->scale[3] * *line1
+			+ program->scale[4] * *line1
+			+ program->scale[5] * *(line1+input->pixelsize)
+			+ program->scale[6] * *line2
+			+ program->scale[7] * *line2
+			+ program->scale[8] * *(line2+input->pixelsize);
+		accu /= program->divisor;
+		_CLAMP65535(accu);
+		*dest = accu;
+		line0++; line1++; line2++; dest++;
+	}
+
+	size = (input->w-1)*input->pixelsize;
+
+	for(col = input->pixelsize; col < size; col++)
+	{
+		accu
+			= program->scale[0] * *(line0-input->pixelsize)
+			+ program->scale[1] * *line0
+			+ program->scale[2] * *(line0+input->pixelsize)
+			+ program->scale[3] * *(line1-input->pixelsize)
+			+ program->scale[4] * *line1
+			+ program->scale[5] * *(line1+input->pixelsize)
+			+ program->scale[6] * *(line2-input->pixelsize)
+			+ program->scale[7] * *line2
+			+ program->scale[8] * *(line2+input->pixelsize);
+		accu /= program->divisor;
+		_CLAMP65535(accu);
+		*dest = accu;
+		line0++; line1++; line2++; dest++;
+	}
+
+	/* special case for last pixel */
+	for (col = size; col < input->w*input->pixelsize; col++)
+	{
+		accu
+			= program->scale[0] * *(line0-input->pixelsize)
+			+ program->scale[1] * *line0
+			+ program->scale[2] * *line0
+			+ program->scale[3] * *(line1-input->pixelsize)
+			+ program->scale[4] * *line1
+			+ program->scale[5] * *line1
+			+ program->scale[6] * *(line2-input->pixelsize)
+			+ program->scale[7] * *line2
+			+ program->scale[8] * *line2;
+		accu /= program->divisor;
+		_CLAMP65535(accu);
+		*dest = accu;
+		line0++; line1++; line2++; dest++;
+	}
 }
 
-inline gushort
-top(gushort *in, struct struct_program *program, gint divisor)
+/**
+ * Concolve a RS_IMAGE16 using a 3x3 kernel
+ * @param input The input image
+ * @param output The output image
+ * @param matrix A 3x3 convolution kernel
+ * @param scaler The result will be scaled like this: convolve/scaler
+ * @return output image for convenience
+ */
+RS_IMAGE16 *
+rs_image16_convolve(RS_IMAGE16 *input, RS_IMAGE16 *output, RS_MATRIX3 *matrix, gfloat scaler, gboolean *abort)
 {
-	gint temp;
-	temp = ((*(in+program[3].offset)) * program[0].scale)
-		+ ((*(in+program[4].offset)) * program[1].scale)
-		+ ((*(in+program[5].offset)) * program[2].scale)
-		+ ((*(in+program[3].offset)) * program[3].scale)
-		+ ((*(in+program[4].offset)) * program[4].scale)
-		+ ((*(in+program[5].offset)) * program[5].scale)
-		+ ((*(in+program[6].offset)) * program[6].scale)
-		+ ((*(in+program[7].offset)) * program[7].scale)
-		+ ((*(in+program[8].offset)) * program[8].scale);
-	temp /= divisor;
-	_CLAMP65535(temp);
-	return(temp);
-}
-
-inline gushort
-topright(gushort *in, struct struct_program *program, gint divisor)
-{
-	gint temp;
-	temp = ((*(in+program[3].offset)) * program[0].scale)
-		+ ((*(in+program[4].offset)) * program[1].scale)
-		+ ((*(in+program[4].offset)) * program[2].scale)
-		+ ((*(in+program[3].offset)) * program[3].scale)
-		+ ((*(in+program[4].offset)) * program[4].scale)
-		+ ((*(in+program[4].offset)) * program[5].scale)
-		+ ((*(in+program[6].offset)) * program[6].scale)
-		+ ((*(in+program[7].offset)) * program[7].scale)
-		+ ((*(in+program[7].offset)) * program[8].scale);
-	temp /= divisor;
-	_CLAMP65535(temp);
-	return(temp);
-}
-
-inline gushort
-left(gushort *in, struct struct_program *program, gint divisor)
-{
-	gint temp;
-	temp = ((*(in+program[1].offset)) * program[0].scale)
-		+ ((*(in+program[1].offset)) * program[1].scale)
-		+ ((*(in+program[2].offset)) * program[2].scale)
-		+ ((*(in+program[4].offset)) * program[3].scale)
-		+ ((*(in+program[4].offset)) * program[4].scale)
-		+ ((*(in+program[5].offset)) * program[5].scale)
-		+ ((*(in+program[7].offset)) * program[6].scale)
-		+ ((*(in+program[7].offset)) * program[7].scale)
-		+ ((*(in+program[8].offset)) * program[8].scale);
-	temp /= divisor;
-	_CLAMP65535(temp);
-	return(temp);
-}
-
-inline gushort
-right(gushort *in, struct struct_program *program, gint divisor)
-{
-	gint temp;
-	temp = ((*(in+program[0].offset)) * program[0].scale)
-		+ ((*(in+program[1].offset)) * program[1].scale)
-		+ ((*(in+program[1].offset)) * program[2].scale)
-		+ ((*(in+program[3].offset)) * program[3].scale)
-		+ ((*(in+program[4].offset)) * program[4].scale)
-		+ ((*(in+program[4].offset)) * program[5].scale)
-		+ ((*(in+program[6].offset)) * program[6].scale)
-		+ ((*(in+program[7].offset)) * program[7].scale)
-		+ ((*(in+program[7].offset)) * program[8].scale);
-	temp /= divisor;
-	_CLAMP65535(temp);
-	return(temp);
-}
-
-inline gushort
-bottomleft(gushort *in, struct struct_program *program, gint divisor)
-{
-	gint temp;
-	temp = ((*(in+program[1].offset)) * program[0].scale)
-		+ ((*(in+program[1].offset)) * program[1].scale)
-		+ ((*(in+program[2].offset)) * program[2].scale)
-		+ ((*(in+program[4].offset)) * program[3].scale)
-		+ ((*(in+program[4].offset)) * program[4].scale)
-		+ ((*(in+program[5].offset)) * program[5].scale)
-		+ ((*(in+program[4].offset)) * program[6].scale)
-		+ ((*(in+program[4].offset)) * program[7].scale)
-		+ ((*(in+program[5].offset)) * program[8].scale);
-	temp /= divisor;
-	_CLAMP65535(temp);
-	return(temp);
-}
-
-inline gushort
-bottom(gushort *in, struct struct_program *program, gint divisor)
-{
-	gint temp;
-	temp = ((*(in+program[0].offset)) * program[0].scale)
-		+ ((*(in+program[1].offset)) * program[1].scale)
-		+ ((*(in+program[2].offset)) * program[2].scale)
-		+ ((*(in+program[3].offset)) * program[3].scale)
-		+ ((*(in+program[4].offset)) * program[4].scale)
-		+ ((*(in+program[5].offset)) * program[5].scale)
-		+ ((*(in+program[3].offset)) * program[6].scale)
-		+ ((*(in+program[4].offset)) * program[7].scale)
-		+ ((*(in+program[5].offset)) * program[8].scale);
-	temp /= divisor;
-	_CLAMP65535(temp);
-	return(temp);
-}
-
-inline gushort
-bottomright(gushort *in, struct struct_program *program, gint divisor)
-{
-	gint temp;
-	temp = ((*(in+program[0].offset)) * program[0].scale)
-		+ ((*(in+program[1].offset)) * program[1].scale)
-		+ ((*(in+program[1].offset)) * program[2].scale)
-		+ ((*(in+program[3].offset)) * program[3].scale)
-		+ ((*(in+program[4].offset)) * program[4].scale)
-		+ ((*(in+program[4].offset)) * program[5].scale)
-		+ ((*(in+program[3].offset)) * program[6].scale)
-		+ ((*(in+program[4].offset)) * program[7].scale)
-		+ ((*(in+program[4].offset)) * program[8].scale);
-	temp /= divisor;
-	_CLAMP65535(temp);
-	return(temp);
-}
-
-inline gushort
-everything(gushort *in, struct struct_program *program, gint divisor)
-{
-	gint temp;
-	temp = ((*(in+program[0].offset)) * program[0].scale)
-		+ ((*(in+program[1].offset)) * program[1].scale)
-		+ ((*(in+program[2].offset)) * program[2].scale)
-		+ ((*(in+program[3].offset)) * program[3].scale)
-		+ ((*(in+program[4].offset)) * program[4].scale)
-		+ ((*(in+program[5].offset)) * program[5].scale)
-		+ ((*(in+program[6].offset)) * program[6].scale)
-		+ ((*(in+program[7].offset)) * program[7].scale)
-		+ ((*(in+program[8].offset)) * program[8].scale);
-	temp /= divisor;
-	_CLAMP65535(temp);
-	return(temp);
-}
-
-RS_IMAGE16
-*rs_image16_convolve(RS_IMAGE16 *input, RS_IMAGE16 *output, RS_MATRIX3 *matrix, gfloat scaler)
-{
-	gint col, row, c;
-	gushort *in, *out;
-	gint divisor = (gint) (scaler * 256.0);
+	gint row;
 	struct struct_program *program;
-	
-	program = (struct struct_program *) g_new(struct struct_program, 9);
-	program[0].offset = -input->rowstride-input->pixelsize;
-	program[1].offset = -input->rowstride;
-	program[2].offset = -input->rowstride+input->pixelsize;
-	program[3].offset = -input->pixelsize;
-	program[4].offset = 0;
-	program[5].offset = input->pixelsize;
-	program[6].offset = input->rowstride-input->pixelsize;
-	program[7].offset = input->rowstride;
-	program[8].offset = input->rowstride+input->pixelsize;
-	program[0].scale = (gint) (matrix->coeff[0][0]*256.0);
-	program[1].scale = (gint) (matrix->coeff[0][1]*256.0);
-	program[2].scale = (gint) (matrix->coeff[0][2]*256.0);
-	program[3].scale = (gint) (matrix->coeff[1][0]*256.0);
-	program[4].scale = (gint) (matrix->coeff[1][1]*256.0);
-	program[5].scale = (gint) (matrix->coeff[1][2]*256.0);
-	program[6].scale = (gint) (matrix->coeff[2][0]*256.0);
-	program[7].scale = (gint) (matrix->coeff[2][1]*256.0);
-	program[8].scale = (gint) (matrix->coeff[2][2]*256.0);
-	
-	if (!output || ((output->w != input->w) || (output->h != input->h))) {
-		return NULL;
-	}
-	
-	/* top left */
-	in = GET_PIXEL(input, 0, 0);
-	out = GET_PIXEL(output, 0, 0);
-	for (c=0;c<input->pixelsize;c++)
-	{
-		*out = topleft(in, program, divisor);
-		in += 1;
-		out += 1;
-	}
-	/* top */
-	in = GET_PIXEL(input, 1, 0);
-	out = GET_PIXEL(output, 1, 0);
-	for(col=1; col<input->w-1; col++)
-	{
-		for (c=0;c<input->pixelsize;c++)
-		{
-			*out = top(in, program, divisor);
-			in += 1;
-			out += 1;
-		}
-	}
-	/* top right */
-	in = GET_PIXEL(input, input->w-1, 0);
-	out = GET_PIXEL(output, input->w-1, 0);
-	for (c=0;c<input->pixelsize;c++)
-	{
-		*out = topright(in, program, divisor);
-		in += 1;
-		out += 1;
-	}
-	/* left border */
-	for(row=1; row<input->h-1; row++)
-	{
-		in = GET_PIXEL(input, 0, row);
-		out = GET_PIXEL(output, 0, row);
-		for (c=0;c<input->pixelsize;c++)
-		{
-			*out = left(in, program, divisor);
-			in += 1;
-			out += 1;
-		}
-	}
-	/* right border */
-	for(row=1; row<input->h-1; row++)
-	{
-		in = GET_PIXEL(input, input->w-1, row);
-		out = GET_PIXEL(output, output->w-1, row);
-		for (c=0;c<input->pixelsize;c++)
-		{
-			*out = right(in, program, divisor);
-			in += 1;
-			out += 1;
-		}
-	}
-	/* bottom left */
-	in = GET_PIXEL(input, 0, input->h-1);
-	out = GET_PIXEL(output, 0, input->h-1);
-	for (c=0;c<input->pixelsize;c++)
-	{
-		*out = bottomleft(in, program, divisor);
-		in += 1;
-		out += 1;
-	}
-	/* bottom */
-	in = GET_PIXEL(input, 1, output->h-1);
-	out = GET_PIXEL(output, 1, output->h-1);
-	for(col=1; col<input->w-1; col++)
-	{
-		for (c=0;c<input->pixelsize;c++)
-		{
-			*out = bottom(in, program, divisor);
-			in += 1;
-			out += 1;
-		}
-	}
-	/* bottom right */
-	in = GET_PIXEL(input, input->w-1, input->h-1);
-	out = GET_PIXEL(output, output->w-1, input->h-1);
 
-	for (c=0;c<input->pixelsize;c++)
+	g_assert(RS_IS_IMAGE16(input));
+	g_assert(RS_IS_IMAGE16(output));
+	g_assert(((output->w == input->w) && (output->h == input->h)));
+
+	rs_image16_ref(input);
+	rs_image16_ref(output);
+
+	/* Make the integer based convolve program */
+	program = (struct struct_program *) g_new(struct struct_program, 1);
+	program->scale[0] = (gint) (matrix->coeff[0][0]*256.0);
+	program->scale[1] = (gint) (matrix->coeff[0][1]*256.0);
+	program->scale[2] = (gint) (matrix->coeff[0][2]*256.0);
+	program->scale[3] = (gint) (matrix->coeff[1][0]*256.0);
+	program->scale[4] = (gint) (matrix->coeff[1][1]*256.0);
+	program->scale[5] = (gint) (matrix->coeff[1][2]*256.0);
+	program->scale[6] = (gint) (matrix->coeff[2][0]*256.0);
+	program->scale[7] = (gint) (matrix->coeff[2][1]*256.0);
+	program->scale[8] = (gint) (matrix->coeff[2][2]*256.0);
+	program->divisor = (gint) (scaler * 256.0);
+
+	for(row = 0; row < input->h; row++)
 	{
-		*out = bottomright(in, program, divisor);
-		in += 1;
-		out += 1;
-	}
-	/* everything but borders */
-	for(row=1; row<input->h-1; row++)
-	{
-		in = GET_PIXEL(input, 1, row);
-		out = GET_PIXEL(output, 1, row);
-		for(col=1; col<input->w-1; col++)
-		{
-			for (c=0;c<input->pixelsize;c++)
-			{
-				gint temp;
-				temp = ((*(in+program[0].offset)) * program[0].scale)
-				+ ((*(in+program[1].offset)) * program[1].scale)
-				+ ((*(in+program[2].offset)) * program[2].scale)
-				+ ((*(in+program[3].offset)) * program[3].scale)
-				+ ((*(in+program[4].offset)) * program[4].scale)
-				+ ((*(in+program[5].offset)) * program[5].scale)
-				+ ((*(in+program[6].offset)) * program[6].scale)
-				+ ((*(in+program[7].offset)) * program[7].scale)
-				+ ((*(in+program[8].offset)) * program[8].scale);
-				
-				temp /= divisor;
-				*out = _CLAMP65535(temp);
-				in += 1;
-				out += 1;
-			}
-		}
+		convolve_line(input, output, row, program);
+		if (abort && *abort) goto abort;
 	}
+
+	g_signal_emit(output, signals[PIXELDATA_CHANGED], 0, NULL);
+
+abort:
+	g_free(program);
+	rs_image16_unref(input);
+	rs_image16_unref(output);
+
 	return output;
 }
 
-
 /**
  * Returns a single pixel from a RS_IMAGE16
  * @param image A RS_IMAGE16
@@ -1286,23 +1049,6 @@ rs_image16_get_pixel(RS_IMAGE16 *image, gint x, gint y, gboolean extend_edges)
 	return pixel;
 }
 
-gboolean
-rs_image16_8_cmp_size(RS_IMAGE16 *a, RS_IMAGE8 *b)
-{
-	gboolean ret = TRUE;
-	rs_image16_ref(a);
-	rs_image8_ref(b);
-	if (!a || !b)
-		ret = FALSE;
-	if (a->w != b->w)
-		ret = FALSE;
-	if (a->h != b->h)
-		ret = FALSE;
-	rs_image16_unref(a);
-	rs_image8_unref(b);
-	return ret;
-}
-
 size_t
 rs_image16_get_footprint(RS_IMAGE16 *image)
 {
@@ -1310,9 +1056,9 @@ rs_image16_get_footprint(RS_IMAGE16 *image)
 }
 
 RS_IMAGE16
-*rs_image16_sharpen(RS_IMAGE16 *in, RS_IMAGE16 *out, gdouble amount)
+*rs_image16_sharpen(RS_IMAGE16 *in, RS_IMAGE16 *out, gdouble amount, gboolean *abort)
 {
-	amount = pow(amount,2)/50;
+	amount = amount/5;
 	
 	gdouble corner = (amount/1.4)*-1.0;
 	gdouble regular = (amount)*-1.0;
@@ -1324,7 +1070,7 @@ RS_IMAGE16
 	if (!out)
 		out = rs_image16_new(in->w, in->h, in->channels, in->pixelsize);
 
-	rs_image16_convolve(in, out, &sharpen, 1);
+	rs_image16_convolve(in, out, &sharpen, 1, abort);
 
 	return out;
 }
@@ -1368,12 +1114,14 @@ rs_image16_copy_double_c(RS_IMAGE16 *in, RS_IMAGE16 *out)
 			i++;
 		}
 	}
+	g_signal_emit(out, signals[PIXELDATA_CHANGED], 0, NULL);
 	rs_image16_unref(in);
 	rs_image16_unref(out);
 
 	return out;
 }
 
+#if defined (__i386__) || defined (__x86_64__)
 RS_IMAGE16 *
 rs_image16_copy_double_mmx(RS_IMAGE16 *in, RS_IMAGE16 *out)
 {
@@ -1447,11 +1195,13 @@ rs_image16_copy_double_mmx(RS_IMAGE16 *in, RS_IMAGE16 *out)
 			: "%"REG_a
 			);
 	}
+	g_signal_emit(out, signals[PIXELDATA_CHANGED], 0, NULL);
 	rs_image16_unref(in);
 	rs_image16_unref(out);
 
 	return out;
 }
+#endif /* defined (__i386__) || defined (__x86_64__) */
 
 /*
 The rest of this file is pretty much copied verbatim from dcraw/ufraw
