@@ -32,6 +32,7 @@
 #include "rs-pixbuf.h"
 #include "eog-pixbuf-cell-renderer.h"
 #include "rs-preload.h"
+#include "rs-photo.h"
 
 /* How many different icon views do we have (tabs) */
 #define NUM_VIEWS 6
@@ -65,17 +66,19 @@ enum {
 
 struct _RSStore
 {
-	GtkNotebook parent;
+	GtkHBox parent;
+	GtkNotebook *notebook;
 	GtkWidget *iconview[NUM_VIEWS];
 	GtkWidget *current_iconview;
 	guint current_priority;
 	GtkListStore *store;
 	gulong counthandler;
 	gchar *last_path;
+	gboolean cancelled;
 };
 
 /* Define the boiler plate stuff using the predefined macro */
-G_DEFINE_TYPE (RSStore, rs_store, GTK_TYPE_NOTEBOOK);
+G_DEFINE_TYPE (RSStore, rs_store, GTK_TYPE_HBOX);
 
 enum {
 	THUMB_ACTIVATED_SIGNAL,
@@ -105,9 +108,11 @@ static void icon_get_selected_iters(GtkIconView *iconview, GtkTreePath *path, gp
 static void icon_get_selected_names(GtkIconView *iconview, GtkTreePath *path, gpointer user_data);
 static gboolean tree_foreach_names(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
 static gboolean tree_find_filename(GtkTreeModel *store, const gchar *filename, GtkTreeIter *iter, GtkTreePath **path);
+static void cancel_clicked(GtkButton *button, gpointer user_data);
+GList *find_loadable(const gchar *path, gboolean load_8bit, gboolean load_recursive);
+void load_loadable(RSStore *store, GList *loadable, RS_PROGRESS *rsp);
 void store_group_select_n(GtkListStore *store, GtkTreeIter iter, guint n);
 gboolean store_iter_is_group(GtkListStore *store, GtkTreeIter *iter);
-gint store_selection_n_groups(RSStore *store, GList *selected);
 void store_save_groups(GtkListStore *store);
 void store_load_groups(GtkListStore *store);
 void store_group_photos_by_iters(GtkListStore *store, GList *members);
@@ -136,11 +141,11 @@ rs_store_class_init(RSStoreClass *klass)
 		G_TYPE_STRING);
 	if (!icon_priority_1)
 	{
-		icon_priority_1	= gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority1.png", NULL);
-		icon_priority_2 = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority2.png", NULL);
-		icon_priority_3 = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority3.png", NULL);
-		icon_priority_D = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_deleted.png", NULL);
-		icon_exported = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_exported.png", NULL);
+		icon_priority_1	= gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/overlay_priority1.png", NULL);
+		icon_priority_2 = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/overlay_priority2.png", NULL);
+		icon_priority_3 = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/overlay_priority3.png", NULL);
+		icon_priority_D = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/overlay_deleted.png", NULL);
+		icon_exported = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/overlay_exported.png", NULL);
 	}
 }
 
@@ -150,14 +155,16 @@ rs_store_class_init(RSStoreClass *klass)
 static void
 rs_store_init(RSStore *store)
 {
-	GtkNotebook *notebook = GTK_NOTEBOOK(store);
+	GtkHBox *hbox = GTK_HBOX(store);
 	gint n;
 	gchar label_text[NUM_VIEWS][63];
 	GtkWidget **label = g_new(GtkWidget *, NUM_VIEWS);
 	GtkWidget *label_tt[NUM_VIEWS];
 	GtkCellRenderer *cell_renderer;
 	gboolean show_filenames;
+	GtkWidget *label_priorities;
 
+	store->notebook = GTK_NOTEBOOK(gtk_notebook_new());
 	store->store = gtk_list_store_new (NUM_COLUMNS,
 		GDK_TYPE_PIXBUF,
 		GDK_TYPE_PIXBUF,
@@ -244,7 +251,7 @@ rs_store_init(RSStore *store)
 		gtk_misc_set_alignment(GTK_MISC(label[n]), 0.0, 0.5);
 
 		/* Add everything to the notebook */
-		gtk_notebook_append_page(notebook, make_iconview(store->iconview[n], store, priorities[n]), label_tt[n]);
+		gtk_notebook_append_page(store->notebook, make_iconview(store->iconview[n], store, priorities[n]), label_tt[n]);
 	}
 
 	/* Load show filenames state from config */
@@ -255,14 +262,25 @@ rs_store_init(RSStore *store)
 	store->current_iconview = store->iconview[0];
 	store->current_priority = priorities[0];
 
-	gtk_notebook_set_tab_pos(notebook, GTK_POS_LEFT);
+	gtk_notebook_set_tab_pos(store->notebook, GTK_POS_LEFT);
 
-	g_signal_connect(notebook, "switch-page", G_CALLBACK(switch_page), store);
+	g_signal_connect(store->notebook, "switch-page", G_CALLBACK(switch_page), store);
 	store->counthandler = g_signal_connect(store->store, "row-changed", G_CALLBACK(count_priorities), label);
 	g_signal_connect(store->store, "row-deleted", G_CALLBACK(count_priorities_del), label);
 
 	all_stores = g_list_append(all_stores, store);
 
+	/* Due to popular demand, I will now add a very nice GTK+ label to the left
+	   of the notebook. We hope this will give our users an even better
+	   understanding of our interface. I was thinking about adding a button instead
+	   that said "ROCK ON!" to instantly play "AC/DC - Highway to Hell", but I
+	   believe this will be better for the end user */
+	label_priorities = gtk_label_new(_("Priorities"));
+	gtk_label_set_angle(GTK_LABEL(label_priorities), 90);
+	gtk_box_pack_start(GTK_BOX (hbox), label_priorities, FALSE, FALSE, 0);
+
+	gtk_box_pack_start(GTK_BOX (hbox), GTK_WIDGET(store->notebook), TRUE, TRUE, 0);
+
 	store->last_path = NULL;
 }
 
@@ -678,6 +696,14 @@ tree_find_filename(GtkTreeModel *store, const gchar *filename, GtkTreeIter *iter
 	return ret;
 }
 
+
+static void
+cancel_clicked(GtkButton *button, gpointer user_data)
+{
+	RSStore *store = RS_STORE(user_data);
+	store->cancelled = TRUE;
+}
+
 /* Public functions */
 
 /**
@@ -727,6 +753,145 @@ rs_store_remove(RSStore *store, const gchar *filename, GtkTreeIter *iter)
 		gtk_list_store_clear(store->store);
 }
 
+/**
+ * Load thumbnails from a directory into the store
+ * @param path The path to load
+ * @param load_8bit Boolean
+ * @param load_recursive Boolean
+ * @return GList containing paths to loadable files
+ */
+GList *
+find_loadable(const gchar *path, gboolean load_8bit, gboolean load_recursive)
+{
+	gchar *name;
+	RS_FILETYPE *filetype;
+	GString *fullname;
+	GList *loadable = NULL;
+	GDir *dir = g_dir_open(path, 0, NULL);
+
+	if (dir == NULL)
+		return loadable;
+
+	while ((name = (gchar *) g_dir_read_name(dir)))
+	{
+		fullname = g_string_new(path);
+		fullname = g_string_append(fullname, G_DIR_SEPARATOR_S);
+		fullname = g_string_append(fullname, name);
+		filetype = rs_filetype_get(name, TRUE);
+		if (filetype)
+		{
+			if (filetype->load && ((filetype->filetype==FILETYPE_RAW)||load_8bit))
+			{
+				loadable = g_list_append(loadable, fullname->str);
+			}
+		}
+		if (load_recursive)
+		{
+			if (g_file_test(fullname->str, G_FILE_TEST_IS_DIR))
+			{
+				/* We don't load hidden directories */
+				if (name[0] != '.')
+				{
+					GList *temp_loadable;
+					temp_loadable = find_loadable(fullname->str, load_8bit, load_recursive);
+					loadable = g_list_concat(loadable, temp_loadable);
+				}
+			}
+		}
+		g_string_free(fullname, FALSE);
+	}
+
+	if (dir)
+		g_dir_close(dir);
+
+	return loadable;
+}
+
+/**
+ * Load thumbnails from a directory into the store
+ * @param store A RSStore
+ * @param loadable A GList containing paths to loadable files
+ * @param rsp A progress bar
+ */
+void
+load_loadable(RSStore *store, GList *loadable, RS_PROGRESS *rsp)
+{
+	gchar *name;
+	RS_FILETYPE *filetype;
+	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf_clean;
+	GdkPixbuf *missing_thumb;
+	gint priority;
+	gboolean exported = FALSE;
+	GtkTreeIter iter;
+	gchar *fullname;
+	gint n;
+
+	/* We will use this, if no thumbnail can be loaded */
+	missing_thumb = gtk_widget_render_icon(GTK_WIDGET(store),
+		GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_DIALOG, NULL);
+
+	for(n = 0; n < g_list_length(loadable); n++)
+	{
+		if (store->cancelled)
+			break;
+		fullname = g_list_nth_data(loadable, n);
+		name = g_path_get_basename(fullname);
+
+		filetype = rs_filetype_get(name, TRUE);
+		if (filetype)
+		{
+			if (filetype->load)
+			{
+				pixbuf = NULL;
+				if (filetype->thumb)
+					pixbuf = filetype->thumb(fullname);
+				if (pixbuf==NULL)
+				{
+					pixbuf = missing_thumb;
+					g_object_ref (pixbuf);
+				}
+
+				/* Save a clean copy of the thumbnail for later use */
+				pixbuf_clean = gdk_pixbuf_copy(pixbuf);
+
+				/* Sane defaults */
+				priority = PRIO_U;
+				exported = FALSE;
+
+				/* Load flags from XML cache */
+				rs_cache_load_quick(fullname, &priority, &exported);
+
+				/* Update thumbnail */
+				thumbnail_update(pixbuf, pixbuf_clean, priority, exported);
+
+				/* Add thumbnail to store */
+				gtk_list_store_prepend (store->store, &iter);
+				gtk_list_store_set (store->store, &iter,
+					PIXBUF_COLUMN, pixbuf,
+					PIXBUF_CLEAN_COLUMN, pixbuf_clean,
+					TEXT_COLUMN, name,
+					FULLNAME_COLUMN, fullname,
+					PRIORITY_COLUMN, priority,
+					EXPORTED_COLUMN, exported,
+					-1);
+
+				/* We can safely unref pixbuf by now, store holds a reference */
+				g_object_unref (pixbuf);
+				g_object_unref (pixbuf_clean);
+
+				/* Move our progress bar */
+				gui_progress_advance_one(rsp);
+			}
+		}
+		g_free(name);
+	}
+
+	g_object_unref(missing_thumb);
+
+	return;
+}
+
 /**
  * Load thumbnails from a directory into the store
  * @param store A RSStore
@@ -736,20 +901,18 @@ rs_store_remove(RSStore *store, const gchar *filename, GtkTreeIter *iter)
 gint
 rs_store_load_directory(RSStore *store, const gchar *path)
 {
-	gchar *name;
-	GtkTreeIter iter;
-	GdkPixbuf *pixbuf;
-	GdkPixbuf *pixbuf_clean;
-	GDir *dir;
+	static gboolean running = FALSE;
+	GStaticMutex lock = G_STATIC_MUTEX_INIT;
+
 	GtkTreeSortable *sortable;
-	gint priority;
-	gboolean exported = FALSE;
-	RS_FILETYPE *filetype;
 	RS_PROGRESS *rsp;
+	GtkWidget *cancel;
 	gboolean load_8bit = FALSE;
+	gboolean load_recursive = DEFAULT_CONF_LOAD_RECURSIVE;
 	gint items=0, n;
 	GtkTreePath *treepath;
-	GdkPixbuf *missing_thumb;
+	GtkTreeIter iter;
+	GList *loadable = NULL;
 
 	g_return_val_if_fail(RS_IS_STORE(store), -1);
 	if (!path)
@@ -766,25 +929,20 @@ rs_store_load_directory(RSStore *store, const gchar *path)
 		store->last_path = g_strdup(path);
 	}
 
-	/* We will use this, if no thumbnail can be loaded */
-	missing_thumb = gtk_widget_render_icon(GTK_WIDGET(store),
-		GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_DIALOG, NULL);
-
-	dir = g_dir_open(path, 0, NULL);
-	if (dir == NULL)
+	/* We should really only be running one instance at a time */
+	g_static_mutex_lock(&lock);
+	if (running)
 		return -1;
+	running = TRUE;
+	g_static_mutex_unlock(&lock);
 
 	rs_conf_get_boolean(CONF_LOAD_GDK, &load_8bit);
+	rs_conf_get_boolean(CONF_LOAD_RECURSIVE, &load_recursive);
 
-	/* Count loadable items */
-	g_dir_rewind(dir);
-	while ((name = (gchar *) g_dir_read_name(dir)))
-	{
-		filetype = rs_filetype_get(name, TRUE);
-		if (filetype)
-			if (filetype->load && ((filetype->filetype==FILETYPE_RAW)||load_8bit))
-				items++;
-	}
+	/* find loadable files */
+	loadable = find_loadable(path, load_8bit, load_recursive);
+	loadable = g_list_first(loadable);
+	items = g_list_length(loadable);
 
 	/* unset model and make sure we have enough columns */
 	for (n=0;n<NUM_VIEWS;n++)
@@ -795,67 +953,18 @@ rs_store_load_directory(RSStore *store, const gchar *path)
 
 	/* Block the priority count */
 	g_signal_handler_block(store->store, store->counthandler);
-	rsp = gui_progress_new_with_delay(NULL, items, 200);
 
-	/* Load all thumbnails */
-	g_dir_rewind(dir);
-	while ((name = (gchar *) g_dir_read_name(dir)))
-	{
-		filetype = rs_filetype_get(name, TRUE);
-		if (filetype)
-			if (filetype->load && ((filetype->filetype==FILETYPE_RAW)||load_8bit))
-			{
-				/* Generate full path to image */
-				GString *fullname;
-				fullname = g_string_new(path);
-				fullname = g_string_append(fullname, G_DIR_SEPARATOR_S);
-				fullname = g_string_append(fullname, name);
+	/* Add a progress bar */
+	store->cancelled = FALSE;
+	cancel = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+	g_signal_connect (G_OBJECT(cancel), "clicked", G_CALLBACK(cancel_clicked), store);
+	rsp = gui_progress_new_with_delay(_("Opening directory..."), items, 200);
+	gui_progress_add_widget(rsp, cancel);
 
-				pixbuf = NULL;
-				if (filetype->thumb)
-					pixbuf = filetype->thumb(fullname->str);
-				if (pixbuf==NULL)
-				{
-					pixbuf = missing_thumb;
-					g_object_ref (pixbuf);
-				}
-
-				/* Save a clean copy of the thumbnail for later use */
-				pixbuf_clean = gdk_pixbuf_copy(pixbuf);
-
-				/* Sane defaults */
-				priority = PRIO_U;
-				exported = FALSE;
-
-				/* Load flags from XML cache */
-				rs_cache_load_quick(fullname->str, &priority, &exported);
-
-				/* Update thumbnail */
-				thumbnail_update(pixbuf, pixbuf_clean, priority, exported);
-
-				/* Add thumbnail to store */
-				gtk_list_store_prepend (store->store, &iter);
-				gtk_list_store_set (store->store, &iter,
-					PIXBUF_COLUMN, pixbuf,
-					PIXBUF_CLEAN_COLUMN, pixbuf_clean,
-					TEXT_COLUMN, name,
-					FULLNAME_COLUMN, fullname->str,
-					PRIORITY_COLUMN, priority,
-					EXPORTED_COLUMN, exported,
-					-1);
-
-				/* We can safely unref pixbuf by now, store holds a reference */
-				g_object_unref (pixbuf);
-
-				/* Don't free the actual data, they're needed in the store */
-				g_string_free(fullname, FALSE);
-
-				/* Move our progress bar */
-				gui_progress_advance_one(rsp);
-			}
-	}
-
-	g_dir_close(dir);
+	/* load all loadable items */
+	load_loadable(store, loadable, rsp);
+	g_list_foreach (loadable, (GFunc)g_free, NULL);
+	g_list_free(loadable);
 
 	/* Sort the store */
 	sortable = GTK_TREE_SORTABLE(store->store);
@@ -897,6 +1006,9 @@ rs_store_load_directory(RSStore *store, const gchar *path)
 	/* Start the preloader */
 	predict_preload(store, TRUE);
 
+	g_static_mutex_lock(&lock);
+	running = FALSE;
+	g_static_mutex_unlock(&lock);
 	/* Return the number of files successfully recognized */
 	return items;
 }
@@ -1264,6 +1376,33 @@ rs_store_select_prevnext(RSStore *store, const gchar *current_filename, guint di
 	return ret;
 }
 
+/**
+ * Switches to the page number page_num
+ * @note Should behave like gtk_notebook_set_current_page()
+ * @param store A RSStore
+ * @param page_num index of the page to switch to, starting from 0. If negative,
+          the last page will be used. If greater than the number of pages in the notebook,
+          nothing will be done.
+ */
+void
+rs_store_set_current_page(RSStore *store, gint page_num)
+{
+	gtk_notebook_set_current_page(store->notebook, page_num);
+}
+
+/**
+ * Returns the page number of the current page.
+ * @note Should behave like gtk_notebook_get_current_page()
+ * @param store A RSStore
+ * @return the index (starting from 0) of the current page in the notebook. If the notebook
+           has no pages, then -1 will be returned.
+ */
+gint
+rs_store_get_current_page(RSStore *store)
+{
+	return gtk_notebook_get_current_page(store->notebook);
+}
+
 GdkPixbuf *
 store_group_update_pixbufs(GdkPixbuf *pixbuf, GdkPixbuf *pixbuf_clean)
 {
@@ -1376,7 +1515,7 @@ rs_store_group_photos(RSStore *store)
 	gint selected_groups;
 
 	selected = rs_store_get_selected_iters(store);
-	selected_groups = store_selection_n_groups(store, selected);
+	selected_groups = rs_store_selection_n_groups(store, selected);
 	gtk_icon_view_unselect_all(GTK_ICON_VIEW(store->current_iconview)); // Or automatic load of photo == wait time
 
 	if (selected_groups == 0)
@@ -1474,7 +1613,7 @@ store_iter_is_group(GtkListStore *store, GtkTreeIter *iter)
 }
 
 gint
-store_selection_n_groups(RSStore *store, GList *selected)
+rs_store_selection_n_groups(RSStore *store, GList *selected)
 {
 	gint n, group = 0;
 
