@@ -26,20 +26,11 @@ typedef struct _RSStoreClass       RSStoreClass;
 
 struct _RSStoreClass
 {
-	GtkNotebookClass parent_class;
+	GtkHBoxClass parent_class;
 };
 
 GType rs_store_get_type (void);
 
-enum {
-	PRIO_U = 0,
-	PRIO_D = 51,
-	PRIO_1 = 1,
-	PRIO_2 = 2,
-	PRIO_3 = 3,
-	PRIO_ALL = 255
-};
-
 /**
  * Creates a new RSStore
  * @return A new GtkWidget
@@ -154,6 +145,27 @@ rs_store_get_name(RSStore *store, GtkTreeIter *iter);
 gboolean
 rs_store_select_prevnext(RSStore *store, const gchar *current_filename, guint direction);
 
+/**
+ * Switches to the page number page_num
+ * @note Should behave like gtk_notebook_set_current_page()
+ * @param store A RSStore
+ * @param page_num index of the page to switch to, starting from 0. If negative,
+          the last page will be used. If greater than the number of pages in the notebook,
+          nothing will be done.
+ */
+extern void
+rs_store_set_current_page(RSStore *store, gint page_num);
+
+/**
+ * Returns the page number of the current page.
+ * @note Should behave like gtk_notebook_get_current_page()
+ * @param store A RSStore
+ * @return the index (starting from 0) of the current page in the notebook. If the notebook
+           has no pages, then -1 will be returned.
+ */
+extern gint
+rs_store_get_current_page(RSStore *store);
+
 /**
  * Marks a selection of thumbnails as a group
  * @param store A RSStore
@@ -168,6 +180,9 @@ rs_store_group_photos(RSStore *store);
 void
 rs_store_ungroup_photos(RSStore *store);
 
+gint
+rs_store_selection_n_groups(RSStore *store, GList *selected);
+
 extern GList *
 rs_store_sort_selected(GList *selected);
 
