@@ -31,9 +31,10 @@
 #include "rs-curve.h"
 #include "rs-preview-widget.h"
 #include "rs-histogram.h"
+#include "rs-photo.h"
 
 /* used for gui_adj_reset_callback() */
-struct reset_carrier {
+struct cb_carrier {
 	RS_BLOB *rs;
 	gint mask;
 };
@@ -49,7 +50,8 @@ static void gui_transform_mirror_clicked(GtkWidget *w, RS_BLOB *rs);
 static void gui_transform_flip_clicked(GtkWidget *w, RS_BLOB *rs);
 static GtkWidget *gui_transform(RS_BLOB *rs, gboolean show);
 static GtkWidget *gui_tool_warmth(RS_BLOB *rs, gint n, gboolean show);
-static gboolean gui_adj_reset_callback(GtkWidget *widget, GdkEventButton *event, struct reset_carrier *rc);
+static gboolean gui_adj_reset_callback(GtkWidget *widget, GdkEventButton *event, struct cb_carrier *rc);
+static void gui_adj_value_callback(GtkRange *range, gpointer user_data);
 static GtkWidget *gui_make_scale_from_adj(RS_BLOB *rs, GCallback cb, GtkObject *adj, gint mask);
 static void curve_context_callback_save(GtkMenuItem *menuitem, gpointer user_data);
 static void curve_context_callback_open(GtkMenuItem *menuitem, gpointer user_data);
@@ -77,41 +79,31 @@ gui_box(const gchar *title, GtkWidget *in, gboolean expanded)
 static void
 gui_transform_rot90_clicked(GtkWidget *w, RS_BLOB *rs)
 {
-	if (!rs->photo) return;
 	rs_photo_rotate(rs->photo, 1, 0.0);
-	rs_update_preview(rs);
 }
 
 static void
 gui_transform_rot180_clicked(GtkWidget *w, RS_BLOB *rs)
 {
-	if (!rs->photo) return;
 	rs_photo_rotate(rs->photo, 2, 0.0);
-	rs_update_preview(rs);
 }
 
 static void
 gui_transform_rot270_clicked(GtkWidget *w, RS_BLOB *rs)
 {
-	if (!rs->photo) return;
 	rs_photo_rotate(rs->photo, 3, 0.0);
-	rs_update_preview(rs);
 }
 
 static void
 gui_transform_mirror_clicked(GtkWidget *w, RS_BLOB *rs)
 {
-	if (!rs->photo) return;
 	rs_photo_mirror(rs->photo);
-	rs_update_preview(rs);
 }
 
 static void
 gui_transform_flip_clicked(GtkWidget *w, RS_BLOB *rs)
 {
-	if (!rs->photo) return;
 	rs_photo_flip(rs->photo);
-	rs_update_preview(rs);
 }
 
 static GtkWidget *
@@ -124,11 +116,11 @@ gui_transform(RS_BLOB *rs, gboolean show)
 	GtkWidget *rot180;
 	GtkWidget *rot270;
 
-	GtkWidget *flip_image = gtk_image_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_flip.png");
-	GtkWidget *mirror_image = gtk_image_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_mirror.png");
-	GtkWidget *rot90_image = gtk_image_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_90.png");
-	GtkWidget *rot180_image = gtk_image_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_180.png");
-	GtkWidget *rot270_image = gtk_image_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_270.png");
+	GtkWidget *flip_image = gtk_image_new_from_file(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/transform_flip.png");
+	GtkWidget *mirror_image = gtk_image_new_from_file(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/transform_mirror.png");
+	GtkWidget *rot90_image = gtk_image_new_from_file(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/transform_90.png");
+	GtkWidget *rot180_image = gtk_image_new_from_file(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/transform_180.png");
+	GtkWidget *rot270_image = gtk_image_new_from_file(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/transform_270.png");
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	flip = gtk_button_new();
@@ -173,8 +165,8 @@ gui_tool_warmth(RS_BLOB *rs, gint n, gboolean show)
 	GtkWidget *wscale;
 	GtkWidget *tscale;
 
-	wscale = gui_make_scale_from_adj(rs, G_CALLBACK(update_preview_callback), rs->settings[n]->warmth, MASK_WARMTH);
-	tscale = gui_make_scale_from_adj(rs, G_CALLBACK(update_preview_callback), rs->settings[n]->tint, MASK_TINT);
+	wscale = gui_make_scale_from_adj(rs, G_CALLBACK(gui_adj_value_callback), rs->settings[n]->warmth, MASK_WARMTH);
+	tscale = gui_make_scale_from_adj(rs, G_CALLBACK(gui_adj_value_callback), rs->settings[n]->tint, MASK_TINT);
 
 	box = gtk_vbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (box), wscale, FALSE, FALSE, 0);
@@ -183,24 +175,35 @@ gui_tool_warmth(RS_BLOB *rs, gint n, gboolean show)
 }
 
 static gboolean
-gui_adj_reset_callback(GtkWidget *widget, GdkEventButton *event, struct reset_carrier *rc)
+gui_adj_reset_callback(GtkWidget *widget, GdkEventButton *event, struct cb_carrier *rc)
 {
 	rs_settings_reset(rc->rs->settings[rc->rs->current_setting], rc->mask);
 	return(TRUE);
 }
 
+static void
+gui_adj_value_callback(GtkRange *range, gpointer user_data)
+{
+	struct cb_carrier *rc = (struct cb_carrier *) user_data;
+
+	if (!rc->rs->mute_signals_to_photo)
+		rs_photo_apply_settings(rc->rs->photo,
+			rc->rs->current_setting,
+			rc->rs->settings[rc->rs->current_setting], rc->mask);
+}
+
 static GtkWidget *
 gui_make_scale_from_adj(RS_BLOB *rs, GCallback cb, GtkObject *adj, gint mask)
 {
 	GtkWidget *hscale, *box, *rimage, *revent;
-	struct reset_carrier *rc = g_malloc(sizeof(struct reset_carrier));
+	struct cb_carrier *rc = g_malloc(sizeof(struct cb_carrier));
 	rc->rs = rs;
 	rc->mask = mask;
 
 	box = gtk_hbox_new(FALSE, 0);
 
 	hscale = gtk_hscale_new((GtkAdjustment *) adj);
-	g_signal_connect(adj, "value_changed", cb, rs);
+	g_signal_connect(adj, "value_changed", cb, rc);
 	gtk_scale_set_value_pos( GTK_SCALE(hscale), GTK_POS_LEFT);
 	gtk_scale_set_digits(GTK_SCALE(hscale), 2);
 
@@ -275,7 +278,7 @@ curve_context_callback_open(GtkMenuItem *menuitem, gpointer user_data)
 	GtkWidget *fc;
 	GString *dir;
 
-	fc = gtk_file_chooser_dialog_new (_("Open Curve ..."), NULL,
+	fc = gtk_file_chooser_dialog_new (_("Open curve ..."), NULL,
 		GTK_FILE_CHOOSER_ACTION_OPEN,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 		GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
@@ -309,8 +312,12 @@ curve_context_callback_reset(GtkMenuItem *menuitem, gpointer user_data)
 {
 	RSCurveWidget *curve = RS_CURVE_WIDGET(user_data);
 
+	gulong handler = g_signal_handler_find(curve, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, update_preview_callback, NULL);
+	g_signal_handler_block(curve, handler);
+
 	rs_curve_widget_reset(curve);
 	rs_curve_widget_add_knot(curve, 0.0,0.0);
+	g_signal_handler_unblock(curve, handler);
 	rs_curve_widget_add_knot(curve, 1.0,1.0);
 }
 
@@ -350,12 +357,7 @@ curve_context_callback(GtkWidget *widget, gpointer user_data)
 static void
 gui_notebook_callback(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, RS_BLOB *rs)
 {
-	rs->current_setting = page_num;
-	if (rs->photo)
-	{
-		rs_update_preview(rs);
-		rs_preview_widget_set_snapshot(RS_PREVIEW_WIDGET(rs->preview), 0, page_num);
-	}
+	rs_set_snapshot(rs, page_num);
 }
 
 void
@@ -409,6 +411,7 @@ make_toolbox(RS_BLOB *rs)
 	GtkWidget **toolbox_contrast = g_new(GtkWidget *, 3);
 	GtkWidget **toolbox_warmth = g_new(GtkWidget *, 3);
 	GtkWidget **toolbox_curve = g_new(GtkWidget *, 3);
+	GtkWidget **toolbox_sharpen = g_new(GtkWidget *, 3);
 	GtkWidget *toolbox_transform;
 	GtkWidget *toolbox_hist;
 	GtkWidget *toolboxscroller;
@@ -428,28 +431,28 @@ make_toolbox(RS_BLOB *rs)
 
 		rs_conf_get_boolean_with_default(CONF_SHOW_TOOLBOX_EXPOSURE, &show, DEFAULT_CONF_SHOW_TOOLBOX_EXPOSURE);
 		toolbox_exposure[n] = gui_box(_("Exposure"), gui_make_scale_from_adj(rs, 
-			G_CALLBACK(update_preview_callback), rs->settings[n]->exposure, MASK_EXPOSURE), show);
+			G_CALLBACK(gui_adj_value_callback), rs->settings[n]->exposure, MASK_EXPOSURE), show);
 		gtk_box_pack_start (GTK_BOX (tbox[n]), toolbox_exposure[n], FALSE, FALSE, 0);
 		g_signal_connect_after(toolbox_exposure[n], "activate", G_CALLBACK(gui_expander_toggle_callback), toolbox_exposure);
 		g_signal_connect_after(toolbox_exposure[n], "activate", G_CALLBACK(gui_expander_save_status_callback), CONF_SHOW_TOOLBOX_EXPOSURE);
 
 		rs_conf_get_boolean_with_default(CONF_SHOW_TOOLBOX_SATURATION, &show, DEFAULT_CONF_SHOW_TOOLBOX_SATURATION);
 		toolbox_saturation[n] = gui_box(_("Saturation"), gui_make_scale_from_adj(rs, 
-			G_CALLBACK(update_preview_callback), rs->settings[n]->saturation, MASK_SATURATION), show);
+			G_CALLBACK(gui_adj_value_callback), rs->settings[n]->saturation, MASK_SATURATION), show);
 		gtk_box_pack_start (GTK_BOX (tbox[n]), toolbox_saturation[n], FALSE, FALSE, 0);
 		g_signal_connect_after(toolbox_saturation[n], "activate", G_CALLBACK(gui_expander_toggle_callback), toolbox_saturation);
 		g_signal_connect_after(toolbox_saturation[n], "activate", G_CALLBACK(gui_expander_save_status_callback), CONF_SHOW_TOOLBOX_SATURATION);
 
 		rs_conf_get_boolean_with_default(CONF_SHOW_TOOLBOX_HUE, &show, DEFAULT_CONF_SHOW_TOOLBOX_HUE);
 		toolbox_hue[n] = gui_box(_("Hue"), gui_make_scale_from_adj(rs, 
-			G_CALLBACK(update_preview_callback), rs->settings[n]->hue, MASK_HUE), show);
+			G_CALLBACK(gui_adj_value_callback), rs->settings[n]->hue, MASK_HUE), show);
 		gtk_box_pack_start (GTK_BOX (tbox[n]), toolbox_hue[n], FALSE, FALSE, 0);
 		g_signal_connect_after(toolbox_hue[n], "activate", G_CALLBACK(gui_expander_toggle_callback), toolbox_hue);
 		g_signal_connect_after(toolbox_hue[n], "activate", G_CALLBACK(gui_expander_save_status_callback), CONF_SHOW_TOOLBOX_HUE);
 
 		rs_conf_get_boolean_with_default(CONF_SHOW_TOOLBOX_CONTRAST, &show, DEFAULT_CONF_SHOW_TOOLBOX_CONTRAST);
 		toolbox_contrast[n] = gui_box(_("Contrast"), gui_make_scale_from_adj(rs,
-			G_CALLBACK(update_preview_callback), rs->settings[n]->contrast, MASK_CONTRAST), show);
+			G_CALLBACK(gui_adj_value_callback), rs->settings[n]->contrast, MASK_CONTRAST), show);
 		gtk_box_pack_start (GTK_BOX (tbox[n]), toolbox_contrast[n], FALSE, FALSE, 0);
 		g_signal_connect_after(toolbox_contrast[n], "activate", G_CALLBACK(gui_expander_toggle_callback), toolbox_contrast);
 		g_signal_connect_after(toolbox_contrast[n], "activate", G_CALLBACK(gui_expander_save_status_callback), CONF_SHOW_TOOLBOX_CONTRAST);
@@ -460,6 +463,13 @@ make_toolbox(RS_BLOB *rs)
 		g_signal_connect_after(toolbox_warmth[n], "activate", G_CALLBACK(gui_expander_toggle_callback), toolbox_warmth);
 		g_signal_connect_after(toolbox_warmth[n], "activate", G_CALLBACK(gui_expander_save_status_callback), CONF_SHOW_TOOLBOX_WARMTH);
 
+		rs_conf_get_boolean_with_default(CONF_SHOW_TOOLBOX_SHARPEN, &show, DEFAULT_CONF_SHOW_TOOLBOX_SHARPEN);
+		toolbox_sharpen[n] = gui_box(_("Sharpen"), gui_make_scale_from_adj(rs,
+			G_CALLBACK(gui_adj_value_callback), rs->settings[n]->sharpen, MASK_SHARPEN), show);
+		gtk_box_pack_start (GTK_BOX (tbox[n]), toolbox_sharpen[n], FALSE, FALSE, 0);
+		g_signal_connect_after(toolbox_sharpen[n], "activate", G_CALLBACK(gui_expander_toggle_callback), toolbox_sharpen);
+		g_signal_connect_after(toolbox_sharpen[n], "activate", G_CALLBACK(gui_expander_save_status_callback), CONF_SHOW_TOOLBOX_SHARPEN);
+
 		rs_conf_get_boolean_with_default(CONF_SHOW_TOOLBOX_CURVE, &show, DEFAULT_CONF_SHOW_TOOLBOX_CURVE);
 		gtk_widget_set_size_request(rs->settings[n]->curve, 64, 64);
 		g_signal_connect(rs->settings[n]->curve, "changed", G_CALLBACK(update_preview_callback), rs);
